@@ -31,6 +31,7 @@ echo ""
 ./abi-extract.pl -N de-CH
 ./abi-extract.pl -N ta-IN
 ./abi-extract.pl -N qu-BO
+./abi-extract.pl -N ast-ES
 
 ./update.pl gl-ES
 ./update.pl nb-NO
@@ -61,3 +62,5 @@ echo ""
 ./update.pl de-CH
 ./update.pl ta-IN
 ./update.pl qu-BO
+./update.pl ast-ES
+
