@@ -41,21 +41,20 @@
 
 /*****************************************************************/
 
-EV_Menu::EV_Menu(EV_EditMethodContainer * pEMC,
+EV_Menu::EV_Menu(XAP_App* pApp,
+				 EV_EditMethodContainer * pEMC,
 				 const char * szMenuLayoutName,
 				 const char * szMenuLabelSetName)
+	: m_pEMC(pEMC),
+	  m_pApp(pApp)
 {
-	UT_ASSERT(pEMC);
-
-	m_pEMC = pEMC;
-
 	//UT_DEBUGMSG(("EV_Menu: Creating menu for [layout %s, language %s]\n",
 	//			 szMenuLayoutName,szMenuLabelSetName));
 	
-	m_pMenuLayout = AP_CreateMenuLayout(szMenuLayoutName);
+	m_pMenuLayout = m_pApp->getMenuFactory()->CreateMenuLayout(szMenuLayoutName);
 	UT_ASSERT(m_pMenuLayout);
 
-	m_pMenuLabelSet = AP_CreateMenuLabelSet(szMenuLabelSetName);
+	m_pMenuLabelSet = m_pApp->getMenuFactory()->CreateMenuLabelSet(szMenuLabelSetName);
 	UT_ASSERT(m_pMenuLabelSet);
 
 }
@@ -66,37 +65,28 @@ EV_Menu::~EV_Menu()
 	DELETEP(m_pMenuLabelSet);
 }
 
-const EV_Menu_Layout * EV_Menu::getMenuLayout() const
-{
-	return m_pMenuLayout;
-}
-
-const EV_Menu_LabelSet * EV_Menu::getMenuLabelSet() const
-{
-	return m_pMenuLabelSet;
-}
-
-void EV_Menu::addPath(const UT_String &path)
+XAP_Menu_Id
+EV_Menu::addMenuItem(const UT_String &path, const UT_String& description)
 {
 	UT_DEBUGMSG(("Adding path %s.\n", path.c_str()));
-	UT_Vector *names = simpleSplit(path, '/');
-	EV_Menu_ActionSet *pMenuActionSet = getApp()->getMenuActionSet();
+	UT_GenericVector<UT_String*> *names = simpleSplit(path, '/');
+//	EV_Menu_ActionSet *pMenuActionSet = getApp()->getMenuActionSet();
 	const UT_String *label;
 	UT_uint32 last_pos = 1;
 	XAP_Menu_Id last_index = 0;
 	XAP_Menu_Id index = 0;
 	UT_ASSERT(names);
 	UT_ASSERT(m_pMenuLabelSet);
-	UT_ASSERT(pMenuActionSet);
+//	UT_ASSERT(pMenuActionSet);
 
 	// if need, we create submenus
 	UT_DEBUGMSG(("Gonna create submenus...\n"));
 	size_t end = names->size() - 1;
 	for (size_t i = 0; i < end; ++i)
 	{
-		label = static_cast<const UT_String*> ((*names)[i]);
+		label = (*names)[i];
 		UT_ASSERT(label);
-		index = EV_searchMenuLabel(*m_pMenuLabelSet, *label);
+		index = EV_searchMenuLabel(m_pMenuLabelSet, *label);
 
 		// Here we should create end - i submenus
 		if (index == 0)
@@ -107,11 +97,11 @@ void EV_Menu::addPath(const UT_String &path)
 			// and now we add the new submenus
 			for (size_t j = i; j < end; ++j)
 			{
-				label = static_cast<const UT_String*> ((*names)[j]);
+				label = (*names)[j];
 				UT_ASSERT(label);
 				index = m_pMenuLayout->addLayoutItem(++lpos, EV_MLF_BeginSubMenu);
-				pMenuActionSet->addAction(new EV_Menu_Action(index, true, false, false, NULL, NULL, NULL));
-				m_pMenuLabelSet->addLabel(new EV_Menu_Label(index, label->c_str(), label->c_str()));
+//				pMenuActionSet->addAction(action);
+				m_pMenuLabelSet->addLabel(new EV_Menu_Label(index, label->c_str(), description.c_str()));
 				_doAddMenuItem(lpos);
 			}
 
@@ -136,21 +126,22 @@ void EV_Menu::addPath(const UT_String &path)
 	UT_DEBUGMSG(("Gonna add a menu item.\n"));
 	// and now we create the menu item
 	index = m_pMenuLayout->addLayoutItem(last_pos, EV_MLF_Normal);
-	pMenuActionSet->addAction(new EV_Menu_Action(index, false, false, false, "scriptPlay", NULL, NULL));
-	m_pMenuLabelSet->addLabel(new EV_Menu_Label(index, ((UT_String *) names->back())->c_str(),
-												((UT_String *) names->back())->c_str()));
+//	pMenuActionSet->addAction(new EV_Menu_Action(index, false, false, false, "scriptPlay", NULL, NULL));
+	m_pMenuLabelSet->addLabel(new EV_Menu_Label(index, names->back()->c_str(),
+												names->back()->c_str()));
 
 	if (!_doAddMenuItem(last_pos))
 	{
 		UT_ASSERT(UT_NOT_IMPLEMENTED);
 #if 0
-		pMenuActionSet->deleteAction(index);
+//		pMenuActionSet->deleteAction(index);
 		m_pMenuLabelSet->deleteLabel(index);
 		m_pMenuLayout->deleteLayoutItem(index);
 #endif
 	}
 
 	delete names;
+	return index;
 }
 
 bool EV_Menu::invokeMenuMethod(AV_View * pView,
@@ -173,15 +164,35 @@ bool EV_Menu::invokeMenuMethod(AV_View * pView,
 	}
 
 	EV_EditMethodCallData emcd(pData,dataLength);
-	(*pEM->getFn())(pView,&emcd);
+	pEM->Fn(pView,&emcd);
 
 	return true;
 }
 
+bool EV_Menu::invokeMenuMethod(AV_View * pView,
+							   EV_EditMethod * pEM,
+							   const UT_String& stScriptName)
+{
+	UT_ASSERT(pEM);
+	EV_EditMethodType t = pEM->getType();
+	if (!(t & EV_EMT_APP_METHOD)) {
+		UT_ASSERT(pView);
+	}
+	if ((t & EV_EMT_REQUIREDATA) && stScriptName.size() == 0)
+	{
+		UT_DEBUGMSG(("    invoke aborted due to lack of script name\n"));
+		return false;
+	}
+
+	EV_EditMethodCallData emcd(stScriptName);
+	pEM->Fn(pView, &emcd);
+
+	return true;
+}
 
 /* replace _ev_GetLabelName () */
 /* this version taken from ev_UnixMenu.cpp */
-const char ** EV_Menu::getLabelName(XAP_App * pApp,  XAP_Frame * pFrame,
+const char ** EV_Menu::getLabelName(XAP_App * pApp, 
 									const EV_Menu_Action * pAction, const EV_Menu_Label * pLabel)
 {
 	static const char * data[2] = {NULL, NULL};
@@ -193,7 +204,7 @@ const char ** EV_Menu::getLabelName(XAP_App * pApp,  XAP_Frame * pFrame,
 	const char * szLabelName;
 	
 	if (pAction->hasDynamicLabel())
-		szLabelName = pAction->getDynamicLabel(pFrame,pLabel);
+		szLabelName = pAction->getDynamicLabel(pLabel);
 	else
 		szLabelName = pLabel->getMenuLabel();
 
@@ -213,7 +224,7 @@ const char ** EV_Menu::getLabelName(XAP_App * pApp,  XAP_Frame * pFrame,
 			EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 			UT_ASSERT(pEM);						// make sure it's bound to something
 
-			const EV_EditEventMapper * pEEM = pFrame->getEditEventMapper();
+			const EV_EditEventMapper * pEEM = getApp()->getEditEventMapper();
 			UT_ASSERT(pEEM);
 
 			const char * string = pEEM->getShortcutFor(pEM);
@@ -226,9 +237,11 @@ const char ** EV_Menu::getLabelName(XAP_App * pApp,  XAP_Frame * pFrame,
 	}
 
 	// set shortcut mnemonic, if any
-	if (*accelbuf)
+	if (*accelbuf) {
+		xxx_UT_DEBUGMSG(("found accelerator %s\n", accelbuf));
 		data[1] = accelbuf;
-	
+	}
+
 	if (!pAction->raisesDialog())
 	{
 		data[0] = szLabelName;
@@ -246,17 +259,19 @@ const char ** EV_Menu::getLabelName(XAP_App * pApp,  XAP_Frame * pFrame,
 	return data;
 }
 
-XAP_Menu_Id EV_searchMenuLabel(const EV_Menu_LabelSet &labels, const UT_String &label)
+XAP_Menu_Id EV_searchMenuLabel(const EV_Menu_LabelSet *labels, const UT_String &label)
 {
-	const UT_Vector &labels_table = labels.getAllLabels();
+	const UT_GenericVector<EV_Menu_Label *> * labels_table = labels->getAllLabels();
 	const EV_Menu_Label *l = 0;
-	UT_uint32 size_labels = labels_table.size();
+
+	UT_ASSERT(labels_table);
+	UT_uint32 size_labels = labels_table->size();
 	XAP_Menu_Id id = 0;
 
 	for (UT_uint32 i = 0; i < size_labels; ++i)
 	{
-		l = static_cast<const EV_Menu_Label *> (labels_table[i]);
-		if (l && label == l->getMenuLabel())
+		l = labels_table->getNthItem(i);
+		if (l && label ==l->getMenuLabel())
 		{
 			id = l->getMenuId();
 			break;
