@@ -35,10 +35,10 @@
 #include "ut_string.h"
 
 /*
- OPTIMIZATIONS:
- -Make BView callbacks for the scroll code that would
-replay a previously recorded BPicture.
- -Get rid of all the sync calls 
+  OPTIMIZATIONS:
+  -Make BView callbacks for the scroll code that would
+  replay a previously recorded BPicture.
+  -Get rid of all the sync calls 
 */
 
 #define DPRINTF(x) 	
@@ -59,7 +59,17 @@ replay a previously recorded BPicture.
 inline uint32 
 utf8_char_len(uchar byte) 
 { 
-  return (((0xE5000000 >> ((byte >> 3) & 0x1E)) & 3) + 1); 
+	return (((0xE5000000 >> ((byte >> 3) & 0x1E)) & 3) + 1); 
+}
+
+const char* GR_Graphics::findNearestFont(const char* pszFontFamily,
+										 const char* pszFontStyle,
+										 const char* pszFontVariant,
+										 const char* pszFontWeight,
+										 const char* pszFontStretch,
+										 const char* pszFontSize)
+{
+	return pszFontFamily;
 }
 
 GR_BeOSGraphics::GR_BeOSGraphics(BView *docview, XAP_App * app) {
@@ -78,8 +88,8 @@ GR_BeOSGraphics::GR_BeOSGraphics(BView *docview, XAP_App * app) {
 	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
 	
 #if defined(USE_BACKING_BITMAP)
-BRect r;
-if (m_pFrontView->Window()->Lock())
+	BRect r;
+	if (m_pFrontView->Window()->Lock())
 	{
 		r = m_pFrontView->Bounds();
 		m_pFrontView->Window()->Unlock();
@@ -105,36 +115,36 @@ if (m_pFrontView->Window()->Lock())
 	m_pFrontView->SetFlags(m_pFrontView->Flags() /*| B_SUBPIXEL_PRECISE*/);
 	m_pFrontView->Window()->Unlock();
 #endif
-/*
- white for _highlight & _bevelup
-        black for foreground
-        lite gray(192*) for background
-        dark gray(128*) for beveldown          
-*/
+	/*
+	  white for _highlight & _bevelup
+	  black for foreground
+	  lite gray(192*) for background
+	  dark gray(128*) for beveldown          
+	*/
 
 	rgb_color c;
 	c.alpha = 255;
 	c.red = c.blue = c.green = 0;		//Black
-        m_3dColors[CLR3D_Foreground] = c;
+	m_3dColors[CLR3D_Foreground] = c;
 	c.red = c.blue = c.green = 219;		//Light Grey
-        m_3dColors[CLR3D_Background] = c;
+	m_3dColors[CLR3D_Background] = c;
 	c.red = c.blue = c.green = 150;		//Dark Grey
-        m_3dColors[CLR3D_BevelDown] = c;
+	m_3dColors[CLR3D_BevelDown] = c;
 	c.red = c.blue = c.green = 255;		//White
-        m_3dColors[CLR3D_Highlight] = c;
+	m_3dColors[CLR3D_Highlight] = c;
 	c.red = c.blue = c.green = 255;		//White
-        m_3dColors[CLR3D_BevelUp] = c;
+	m_3dColors[CLR3D_BevelUp] = c;
 }		
 
 GR_BeOSGraphics::~GR_BeOSGraphics() {
 #if defined(USE_BACKING_BITMAP)
 	if (!m_pShadowBitmap)
 		return;
-if (m_pShadowBitmap->Lock())
-{
-	m_pShadowBitmap->RemoveChild(m_pShadowView);
-	m_pShadowBitmap->Unlock();
-}
+	if (m_pShadowBitmap->Lock())
+	{
+		m_pShadowBitmap->RemoveChild(m_pShadowView);
+		m_pShadowBitmap->Unlock();
+	}
 	delete m_pShadowBitmap;
 	delete m_pShadowView;
 #endif
@@ -175,21 +185,22 @@ bool GR_BeOSGraphics::queryProperties(GR_Graphics::Properties gp) const
 {
 	switch (gp)
 	{
-	case DGP_SCREEN:
-		return true;
-	case DGP_PAPER:			//Not sure what this does
-		return true;
-		return false;
-	default:
-		UT_ASSERT(0);
-		return false;
+		case DGP_OPAQUEOVERLAY:
+		case DGP_SCREEN:
+			return true;
+		case DGP_PAPER:			//Not sure what this does
+			return false;
+		default:
+			UT_ASSERT(0);
+			return false;
 	}
 }
 
 
 // Draw this string of characters on the screen in current font
 void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
-							    int iLength, UT_sint32 xoff, UT_sint32 yoff)
+							    int iLength, UT_sint32 xoff, UT_sint32 yoff,
+								int * pCharWidths)
 {
 	int i;
 	char buffer[2*(iLength+1)];
@@ -197,7 +208,7 @@ void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
     buffer[0] = '\0';
     char* builder=buffer;
 	for (i=0; i<iLength; ++i) {
-		char* utf8buf = UT_encodeUTF8char(remapGlyph(pChars[i+iCharOffset], false));
+		char* utf8buf = UT_encodeUTF8char(pChars[i+iCharOffset]);
 		int len = utf8_char_len(*utf8buf);
 		memcpy(builder, utf8buf, len);
 		builder+=len; 						
@@ -239,8 +250,7 @@ void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	viewFont.GetEscapements(buffer,iLength,&tempdelta,escapementArray);
 
 
-	// TODO: need remapGlyph() before the following function call?
-	m_pShadowView->DrawString(UT_encodeUTF8char(remapGlyph(pChars[0+iCharOffset], false)),
+	m_pShadowView->DrawString(UT_encodeUTF8char(pChars[0+iCharOffset]),
 							  BPoint(xoff,yoff+offset));
 								  
 	for (i=1; i<iLength; i++)
@@ -251,7 +261,7 @@ void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 		 */
 		widthAbiWants=(unsigned short int)(escapementArray[i-1].x*fontsize);
 		xoff+=widthAbiWants;
-		m_pShadowView->DrawString(UT_encodeUTF8char(remapGlyph(pChars[i+iCharOffset], false)),
+		m_pShadowView->DrawString(UT_encodeUTF8char(pChars[i+iCharOffset]),
 								  BPoint(xoff,yoff+offset));
 	}
 
@@ -263,6 +273,11 @@ void GR_BeOSGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 	UPDATE_VIEW
 }
 
+void GR_BeOSGraphics::drawGlyph(UT_uint32 Char,UT_sint32 xoff,UT_sint32 yoff)
+{
+	UT_ASSERT(UT_TODO);
+}
+
 BFont *findClosestFont(const char* pszFontFamily, 
 		 	const char* pszFontStyle, 
 			const char* pszFontWeight) {
@@ -367,7 +382,7 @@ GR_Font* GR_BeOSGraphics::findFont(const char* pszFontFamily,
 	BFont 	*aFont;
 	//int 	size = atoi(pszFontSize);
 	//UT_sint32 iHeight = convertDimension(pszFontSize);
-	int		size = convertDimension(pszFontSize);
+	double	size = UT_convertToPoints(pszFontSize);
 
 	DPRINTF(printf("GR Find Font:\n\tFamily: %s ", pszFontFamily));
 	DPRINTF(printf("\n\tStyle: %s ", pszFontStyle));
@@ -477,9 +492,9 @@ UT_uint32 GR_BeOSGraphics::getFontHeight(GR_Font *font)
 	return((UT_uint32)(fh.ascent + fh.descent + fh.leading + 0.5));
 }
 
-UT_uint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
+UT_sint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
 {
-	//We need to convert the string from UCS2 to UTF8 before
+	//We need to convert the string from UCS-X to UTF-8 before
 	//we use the BeOS string operations on it.
 	char buffer[10];
 
@@ -502,7 +517,7 @@ UT_uint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
 
 		m_pShadowView->SetFont(&viewFont);
 		
-		//Hope this works on UTF8 characters buffers
+		//Hope this works on UTF-8 characters buffers
 		viewFont.GetEscapements(buffer,1,&tempdelta,escapementArray);
 		fontsize=viewFont.Size();
 
@@ -514,14 +529,25 @@ UT_uint32 GR_BeOSGraphics::measureUnRemappedChar(const UT_UCSChar c)
 	return 1.0f; // Shouldn't happen.
 }
 
+void GR_BeOSGraphics::getCoverage(UT_Vector& coverage)
+{
+	UT_ASSERT(UT_TODO);
+}
+
 UT_uint32 GR_BeOSGraphics::_getResolution() const
 {
 	return 72;
 }
 
+void GR_BeOSGraphics::getColor(UT_RGBColor& clr)
+{
+	clr = m_curColor;
+}
+
 void GR_BeOSGraphics::setColor(const UT_RGBColor& clr)
 {
 	DPRINTF(printf("GR: setColor\n"));
+	m_curColor = clr;
 	if (m_pShadowView->Window()->Lock())
 	{
 		m_pShadowView->SetHighColor(clr.m_red, clr.m_grn, clr.m_blu);
@@ -605,12 +631,8 @@ void GR_BeOSGraphics::invertRect(const UT_Rect* pRect)
 	UPDATE_VIEW
 }
 
-void GR_BeOSGraphics::fillRect(UT_RGBColor& c, UT_Rect &r) {
-	fillRect(c,r.left,r.top,r.width,r.height);
-}
 
-
-void GR_BeOSGraphics::fillRect(UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
+void GR_BeOSGraphics::fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 						UT_sint32 w, UT_sint32 h)
 {
 	DPRINTF(printf("GR: Flll Rect\n"));
@@ -901,12 +923,10 @@ void GR_BeOSGraphics::setCursor(GR_Graphics::Cursor c)
 		cursor_number = GDK_SB_RIGHT_ARROW; //GDK_ARROW;
 		break;
 
-#ifdef BIDI_ENABLED
 //#error choose a suitable cursor; this is just a placeholder !!!		
 	case GR_CURSOR_LEFTARROW:
 		cursor_number = GDK_SB_LEFT_ARROW; //GDK_ARROW;
 		break;
-#endif
 	
 	case GR_CURSOR_IMAGE:
 		cursor_number = GDK_FLEUR;
@@ -956,6 +976,28 @@ GR_Graphics::Cursor GR_BeOSGraphics::getCursor(void) const
 	return m_cursor;
 }
 
+void GR_BeOSGraphics::saveRectangle(UT_Rect & r, UT_uint32 iIndx)
+{
+	UT_ASSERT(UT_TODO);
+}
+
+void GR_BeOSGraphics::restoreRectangle(UT_uint32 iIndx)
+{
+	UT_ASSERT(UT_TODO);
+}
+
+UT_uint32 GR_BeOSGraphics::getDeviceResolution(void) const
+{
+	BScreen * pScreen = new BScreen(m_pShadowView->Window());
+	if (pScreen)
+	{
+		BRect r = pScreen->Frame();
+		return (UT_uint32)(r.Width()); // NOTE: assumes square pixels
+	}
+	else
+		return 800; // fallback
+}
+
 void GR_BeOSGraphics::setColor3D(GR_Color3D c)
 {
 	DPRINTF(printf("Set color 3D %d \n", c));
@@ -1012,3 +1054,33 @@ void GR_Font::s_getGenericFontProperties(const char * szFontName,
 	*pfp = FP_Unknown;
 	*pbTrueType = true;
 }
+
+UT_sint32 BeOSFont::measureUnremappedCharForCache(UT_UCSChar cChar) const
+{
+	// first of all, handle 0-width spaces ...
+	if(cChar == 0xFEFF || cChar == 0x200b || cChar == UCS_LIGATURE_PLACEHOLDER)
+		return 0;
+
+ 	//We need to convert the string from UCS-X to UTF-8 before
+ 	//we use the BeOS string operations on it.
+ 	char buffer[10];
+ 
+ 	BPoint escapementArray[1];
+ 	
+ 	char * utf8char;
+ 	utf8char =  UT_encodeUTF8char(cChar);
+ 	strcpy(buffer, utf8char);						
+ 
+ 	escapement_delta tempdelta;
+ 	tempdelta.space=0.0;
+ 	tempdelta.nonspace=0.0;
+ 	float fontsize=0.0f;
+ 
+ 	m_pBFont->SetSpacing(B_BITMAP_SPACING);
+ 
+ 	//Hope this works on UTF-8 characters buffers
+ 	m_pBFont->GetEscapements(buffer,1,&tempdelta,escapementArray);
+ 	fontsize=m_pBFont->Size();
+ 
+ 	return (UT_uint32)(escapementArray[0].x *fontsize);
+}
