@@ -20,15 +20,29 @@
 #ifndef UT_LANGUAGE_H
 #define UT_LANGUAGE_H
 
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
-#include "ut_xml.h"
+#endif
+
+enum UT_LANGUAGE_DIR
+{
+	UTLANG_LTR,
+	UTLANG_RTL,
+	UTLANG_VERTICAL
+};
+
+struct UT_LangRecord
+{
+	XML_Char * m_szLangCode;
+	XML_Char * m_szLangName;
+	UT_uint32  m_nID;
+    UT_LANGUAGE_DIR m_eDir;
+};
 
-typedef struct
-	{
-		XML_Char * prop;
-		XML_Char * lang;
-		UT_uint32  id;
-	} lang_entry;
 
 class ABI_EXPORT UT_Language
 {
@@ -36,12 +50,16 @@ public:
 	UT_Language();
 
 	UT_uint32	getCount();
-	const XML_Char * 	getNthProperty(UT_uint32 n);
-	const XML_Char * 	getNthLanguage(UT_uint32 n);
-	const XML_Char * 	getPropertyFromLanguage(const XML_Char * lang);
-	const XML_Char * 	getPropertyFromProperty(const XML_Char * prop); //see the cpp file for explanation
-	UT_uint32 	getIndxFromProperty(const XML_Char * prop);
-	UT_uint32 	getIdFromProperty(const XML_Char * prop);
+	const XML_Char * 	getNthLangCode(UT_uint32 n);
+	const XML_Char * 	getNthLangName(UT_uint32 n);
+	const UT_uint32  	getNthId(UT_uint32 n);
+	const XML_Char * 	getCodeFromName(const XML_Char * szName);
+	const XML_Char * 	getCodeFromCode(const XML_Char * szCode); //
+																  //see the cpp file for explanation
+	const UT_LangRecord* getLangRecordFromCode(const XML_Char * szCode);
+	UT_uint32 	        getIndxFromCode(const XML_Char * szCode);
+	UT_uint32 	        getIdFromCode(const XML_Char * szCode);
+	UT_LANGUAGE_DIR		getDirFromCode(const XML_Char * szCode);
 
 private:
 	static bool	s_Init;
