@@ -237,6 +237,9 @@ void XAP_CocoaFrameImpl::_nullUpdate() const
 	[[m_frameController window] displayIfNeeded];
 }
 
+static UT_uint32	s_iNewFrameOffsetX = 0;
+static UT_uint32	s_iNewFrameOffsetY = 0;
+
 void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 {
 	// create a top-level window for us.
@@ -276,13 +279,20 @@ void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 
 	if (f & XAP_CocoaApp::GEOMETRY_FLAG_SIZE)
 	{
-		NSRect windowFrame;
 		NSRect screenFrame = [[NSScreen mainScreen] visibleFrame];
 
-		windowFrame.size.width = UT_MIN(screenFrame.size.width - 30, 813);
+		screenFrame.origin.x    += s_iNewFrameOffsetX;
+		screenFrame.size.height -= s_iNewFrameOffsetY;
+
+		s_iNewFrameOffsetX = (s_iNewFrameOffsetX <= 128) ? (s_iNewFrameOffsetX + 32) : 0;
+		s_iNewFrameOffsetY = (s_iNewFrameOffsetY <  128) ? (s_iNewFrameOffsetY + 32) : 0;
+
+		NSRect windowFrame;
+		windowFrame.size.width  = UT_MIN(screenFrame.size.width - 30, 813);
 		windowFrame.size.height = UT_MIN(screenFrame.size.height, 836);
 		windowFrame.origin.x = screenFrame.origin.x;
 		windowFrame.origin.y = screenFrame.size.height - windowFrame.size.height;
+
 		[theWindow setFrame:windowFrame display:YES];
 	}
 
@@ -304,35 +314,6 @@ void XAP_CocoaFrameImpl::_createTopLevelWindow(void)
 	return;
 }
 
-/*!
- * This code is used by the dynamic menu API to rebuild the menus after a
- * a change in the menu structure.
- */
-void XAP_CocoaFrameImpl::rebuildMenus(void)
-{
-	UT_ASSERT_NOT_REACHED();
-#if 0
-//
-// Destroy the old menu bar
-//
-	m_pCocoaMenu->destroy();
-//
-// Delete the old class
-//
-	DELETEP(m_pCocoaMenu);
-//
-// Build a new one.
-//
-	m_pCocoaMenu = new EV_CocoaMenuBar(m_pCocoaApp,(AP_CocoaFrame*)getFrame(),
-									 m_szMenuLayoutName,
-									 m_szMenuLabelSetName);
-	UT_ASSERT(m_pCocoaMenu);
-	bool bResult = m_pCocoaMenu->rebuildMenuBar();
-	UT_ASSERT(bResult);
-#endif
-}
-
-
 /*!
  * This code is used by the dynamic toolbar API to rebuild a toolbar after a
  * a change in the toolbar structure.
@@ -532,7 +513,14 @@ void XAP_CocoaFrameImpl::setToolbarRect(const NSRect &r)
 		if (pEM->Fn(m_frame->getFrame()->getCurrentView(),NULL))
 		{
 			// returning YES means close the window
+			// but first let's warn the app controller not to assume our existence
 			
+			XAP_Frame * pFrame = m_frame->getFrame ();
+			AV_View * pView = pFrame->getCurrentView();
+
+			XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+			[pController unsetCurrentView:pView inFrame:pFrame];
+
 			return YES;
 		}
 	}
@@ -564,8 +552,6 @@ void XAP_CocoaFrameImpl::setToolbarRect(const NSRect &r)
 	XAP_Frame * pFrame = m_frame->getFrame ();
 	AV_View * pView = pFrame->getCurrentView();
 
-	XAP_App::getApp()->rememberFocussedFrame (static_cast<void *>(pFrame));
-
 	XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
 	[pController setCurrentView:pView inFrame:pFrame];
 
@@ -584,10 +570,6 @@ void XAP_CocoaFrameImpl::setToolbarRect(const NSRect &r)
 {
 	UT_DEBUGMSG(("windowDidResignKey: '%s'\n", [[[self window] title] UTF8String]));
 
-	XAP_Frame * frame = m_frame->getFrame ();
-	XAP_App * App = frame->getApp ();
-	App->clearLastFocussedFrame ();
-
 	XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
 	[pController setCurrentView:0 inFrame:0];
 
