@@ -21,18 +21,32 @@
 /* XAP_CocoaToolbarWindow */
 
 #import <Cocoa/Cocoa.h>
+#include "ut_vector.h"
 
 class EV_CocoaToolbar;
+@class XAP_CocoaFrameController;
 
-@interface XAP_CocoaToolbarWindow : NSWindowController
+@interface XAP_CocoaToolbarWindow_Controller : NSWindowController
 {
-	IBOutlet NSView *m_topView;
+	id			m_current;
+	UT_Vector * m_toolbarVector;
+	BOOL		m_lock;
 }
-+ (XAP_CocoaToolbarWindow *)sharedToolbar;
-+ (XAP_CocoaToolbarWindow *)createFromNib;
-- (void)windowDidLoad;
++ (NSRect)defaultFrame:(float)height;
++ (XAP_CocoaToolbarWindow_Controller *)sharedToolbar;
++ (XAP_CocoaToolbarWindow_Controller *)create;
+- (id)initWithWindow:(NSWindow *)window;
+- (void)dealloc;
 - (void)removeAllToolbars;
-- (BOOL)addToolbar:(EV_CocoaToolbar *)aToolbar;
-- (BOOL)removeToolbar:(EV_CocoaToolbar *)aToolbar;
-- (NSView *)getTopView;
+- (void)redisplayToolbars:(XAP_CocoaFrameController*)frame;
+- (void)autoResize;
+
+/* lock and unlock redraw for batch toolbar changes, at initialization */
+- (void)lock;
+- (void)unlock;
+
+- (void)_showAllToolbars:(XAP_CocoaFrameController*)frame;
+
+- (void)showToolbarNotification:(NSNotification*)notif;
+- (void)hideToolbarNotification:(NSNotification*)notif;
 @end
