@@ -36,6 +36,9 @@
 #include "ie_impGraphic.h"
 #include "fv_View.h"
 #include "ap_Win32App.h"
+#include "fg_Graphic.h"
+#include "gr_Win32Graphics.h"
+
 
 #ifdef _MSC_VER
 #pragma warning(disable: 4355)	// 'this' used in base member initializer list
@@ -63,8 +66,6 @@
 
 WHICHPROC s_oldRedBar; 
 
-extern "C" const char * wvLIDToLangConverter(unsigned short langID);
-
 // Where the heck is this function????
 // TODO Fix the following header file. It seems to be incomplete
 // TODO #include <ap_EditMethods.h>
@@ -76,7 +77,7 @@ extern XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFra
 */
 LRESULT CALLBACK s_rebarWndProc( HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
 {
-	switch (uMessage)	
+	switch (uMessage)
 	{		
 		case WM_DRAWITEM:
 		{
@@ -130,6 +131,11 @@ XAP_Win32FrameImpl::~XAP_Win32FrameImpl(void)
 	
 	DELETEP(m_pWin32Menu);
 	DELETEP(m_pWin32Popup);
+
+	// have to reset the window long, so our message functions do not
+	// try to derefernce it
+	SetWindowLong(m_hwndFrame, GWL_USERDATA,0);
+
 }
 
 
@@ -168,7 +174,7 @@ void XAP_Win32FrameImpl::_initialize(void)
 	// get a handle to our keyboard binding mechanism
 	// and to our mouse binding mechanism.
 	
-	EV_EditEventMapper * pEEM = getFrame()->getEditEventMapper();
+	EV_EditEventMapper * pEEM = XAP_App::getApp()->getEditEventMapper();
 	UT_return_if_fail(pEEM);
 
 	m_pKeyboard = new ev_Win32Keyboard(pEEM);
@@ -234,7 +240,7 @@ void XAP_Win32FrameImpl::_createTopLevelWindow(void)
 	// synthesize a menu from the info in our
 	// base class and install it into the window.
 	m_pWin32Menu = new EV_Win32MenuBar(pWin32App,
-							getFrame()->getEditEventMapper(),
+							 XAP_App::getApp()->getEditEventMapper(),
 							m_szMenuLayoutName,
 							m_szMenuLabelSetName);
 	UT_ASSERT(m_pWin32Menu);
@@ -317,7 +323,6 @@ bool XAP_Win32FrameImpl::_close(void)
 	// last window closed is the one that the window state is stored from.
 	WINDOWPLACEMENT wndPlacement;
 	wndPlacement.length = sizeof(WINDOWPLACEMENT); // must do
-	
 	if (GetWindowPlacement(m_hwndFrame, &wndPlacement))
 	{
 		UT_uint32 nFlags = PREF_FLAG_GEOMETRY_POS | PREF_FLAG_GEOMETRY_SIZE;
@@ -378,7 +383,9 @@ EV_Toolbar * XAP_Win32FrameImpl::_newToolbar(XAP_App *app, XAP_Frame *frame, con
 	// for now, position each one manually
 	// TODO: put 'em all in a rebar instead
 	HWND hwndBar = result->getWindow();
-	
+	if (!hwndBar) 
+		return result;
+
 	RECT rcClient;
 	GetClientRect(hwndBar, &rcClient);
 	const UT_uint32 iHeight = rcClient.bottom - rcClient.top;
@@ -437,90 +444,45 @@ bool XAP_Win32FrameImpl::_runModalContextMenu(AV_View * pView, const char * szMe
 
 void XAP_Win32FrameImpl::_setFullScreen(bool isFullScreen)
 {
-	// TODO: currently does nothing
-}
+	XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
+	HWND hwndFrame = static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow();
 
+	// Get the window's style so we can add or remove the titlebar later
+	long hStyle = GetWindowLong(hwndFrame, GWL_STYLE);
 
-bool XAP_Win32FrameImpl::_openURL(const char * szURL)
-{
-	// NOTE: could get finer control over browser window via DDE 
-	// NOTE: may need to fallback to WinExec for old NSCP versions
-
-	UT_String sURL = szURL;
-
-	// strip file:///\ from URL, the extra forward-slash back-slash is passed in from "View as Web Page"
-	if ( "file:///\\" == sURL.substr(0,9) )
+	if (isFullScreen)
 	{
-		sURL = sURL.substr(9, sURL.size() - 9);
-	}
-
-	// strip "file://" from URL, win32 doesn't handle them well
-	if ( "file://" == sURL.substr(0, 7) )
-	{
-		sURL = sURL.substr(7, sURL.size() - 7);
-	}
-	
-	/* */
-	for (unsigned int i=0; i<sURL.length();i++)	
-		if (sURL[i]=='\\')	sURL[i]='/';		
-	
-
-	int res = (int) ShellExecute(m_hwndFrame /*(HWND) top level window */, "open", sURL.c_str(), NULL, NULL, SW_SHOWNORMAL);
-
-	// TODO: localized error messages
-	// added more specific error messages as documented in http://msdn.microsoft.com/library/default.asp?url=/library/en-us/debug/base/system_error_codes.asp
-
-	if (res <= 32)	// show error message if failed to launch browser to display URL
-	{
-		UT_String errMsg;
-		switch (res)
+		// Save window state prior to fullscreen (are we maximized?)
+		WINDOWPLACEMENT wndPlacement;
+		wndPlacement.length = sizeof(WINDOWPLACEMENT);
+		if (GetWindowPlacement(hwndFrame, &wndPlacement))
 		{
-			case 2:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: The system cannot find the file specified.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
+			m_iWindowStateBeforeFS = wndPlacement.showCmd;
+		}
+		else
+		{
+			// Couldn't retrieve windowplacement info
+			// Assume we were at Normal (non-maximized) state
+			m_iWindowStateBeforeFS = SW_SHOWNORMAL;
+		}
+	}
 
-			case 3:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: The system cannot find the path specified.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
+	// Add or remove title-bar and border
+	SetWindowLong(hwndFrame, GWL_STYLE, isFullScreen ? hStyle & ~WS_CAPTION : hStyle | WS_CAPTION);
 
-			case 5:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: Access is denied.\n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
+	// We hide the window before maximizing
+	// to ensure it displays with the proper geometry
+	// and to bypass the window resizing animation
+	ShowWindow(hwndFrame, SW_HIDE);
 
-			default:
-				{
-					errMsg = "Error ("; 
-					errMsg += UT_String_sprintf("%d", res);
-					errMsg += ") displaying URL: \n";
-					errMsg += " [ ";  errMsg += sURL;  errMsg += " ] ";
-					MessageBox(m_hwndFrame, errMsg.c_str(), "Error displaying URL", MB_OK|MB_ICONEXCLAMATION);
-				}
-				break;
-		} /* switch (res) */
-	} /* if (res <= 32) */
+	ShowWindow(hwndFrame, isFullScreen ? SW_SHOWMAXIMIZED : m_iWindowStateBeforeFS);
 
-	return (res>32);
+	UpdateWindow(hwndFrame);
 }
 
 
+
+
 void XAP_Win32FrameImpl::_nullUpdate (void) const
 {
 	MSG msg;
@@ -535,7 +497,16 @@ void XAP_Win32FrameImpl::_nullUpdate (void) const
 
 void XAP_Win32FrameImpl::_setCursor(GR_Graphics::Cursor cursor)
 {
-	// TODO: currently does nothing
+	FV_View* pView = (FV_View *) getFrame()->getCurrentView();
+
+	// this is legitimate, when we first load, the frame has no view
+	if(!pView)
+		return;
+	
+	GR_Win32Graphics * pG = (GR_Win32Graphics*)pView->getGraphics();
+	UT_return_if_fail( pG );
+
+	pG->handleSetCursorMessage();
 }
 
 UT_RGBColor XAP_Win32FrameImpl::getColorSelBackground () const
@@ -611,9 +582,30 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 	switch (iMsg)
 	{
 	
-	
-	case WM_EXITMENULOOP:
 	case WM_SETFOCUS:
+		{
+			// on set focus we want to make sure that the kbd layout
+			// is the same as it was before we lost focus. If the user
+			// was in some other application and changed the layout in
+			// there to something else and now returns to AW then we
+			// should not be in that different layout (this should not
+			// happen as the layouts are supposed to be thread
+			// specific, but I noted that sometimes even thought we
+			// have the correct layout, the icon on the task bar is
+			// out of sync with our layout; this should fix that).
+			XAP_Win32App *pWin32App = static_cast<XAP_Win32App *>(XAP_App::getApp());
+			UT_ASSERT( pWin32App );
+
+			if(pWin32App)
+			{
+				ActivateKeyboardLayout(pWin32App->getHKL(),0);
+			}
+			
+			
+			// fall through ...
+		}
+			
+	case WM_EXITMENULOOP:
 		if (pView)
 		{
 			pView->focusChange(AV_FOCUS_HERE);
@@ -803,11 +795,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 								hWndChild = FindWindowEx( pNMcd->hdr.hwndFrom, hWndChild, NULL, NULL );
 							}
 
-							if( hBr != NULL )
-							{
-								DeleteObject( hBr );
-								hBr = NULL;
-							}
+							/* Don't delete hBr since it was obtained using GetSysColorBrush, so System owned */
 						}
 						break;
 					}
@@ -886,47 +874,20 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	case WM_INPUTLANGCHANGE:
 	{
-		UT_DEBUGMSG(("Frame received input language change\n"));
+		UT_DEBUGMSG(("XAP_Win32FrameImpl::_FrameWndProc: received input language change\n"));
 
 		// This will remap the static tables used by all frames.
 		// (see the comment in ev_Win32Keyboard.cpp.)
 		ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(fimpl->m_pKeyboard);
 		pWin32Keyboard->remapKeyboard((HKL)lParam);
 
-		// we also want to automatically change the language at
-		// the insertion point, unless the user preferences tell us
-		// otherwise
+		// set the language
+		XAP_Win32App *pWin32App = static_cast<XAP_Win32App *>(XAP_App::getApp());
+		UT_ASSERT( pWin32App );
 
-	    bool bChangeLang = true;
-		XAP_App::getApp()->getPrefsValueBool(XAP_PREF_KEY_ChangeLanguageWithKeyboard, &bChangeLang);
+		if(pWin32App)
+			pWin32App->setKbdLanguage((HKL)lParam);
 
-		if(bChangeLang)
-		{
-			WORD langID = LANGIDFROMLCID(LOWORD((HKL)lParam));
-			const char * pszLang = wvLIDToLangConverter((unsigned short)langID);
-			UT_DEBUGMSG(("Keyboard Language: %s\n",pszLang));
-			
-			// now invoke the appropriet formatting method ...
-			EV_EditMethodCallData CallData(pszLang,strlen(pszLang));
-
-			XAP_App * pApp = f->getApp();
-			UT_ASSERT(pApp);
-
-			const EV_EditMethodContainer * pEMC = pApp->getEditMethodContainer();
-			UT_ASSERT(pEMC);
-
-			if(pEMC)
-			{
-				EV_EditMethod * pEM = pEMC->findEditMethodByName("language");
-				UT_ASSERT(pEM);						// make sure it's bound to something
-			
-				if (pEM)
-				{
-					pEM->Fn(pView,&CallData);
-				}
-			}
-		}
-		
 		// Do not propagate this message.
 		
 		return 1; //DefWindowProc(hwnd, iMsg, wParam, lParam);
@@ -1002,7 +963,7 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 						DELETEP(pIEG);					
 
-						errorCode = pView->cmdInsertGraphic(pFG, szFileName);
+						errorCode = pView->cmdInsertGraphic(pFG);
 						if (errorCode != UT_OK)
 						{
 							s_CouldNotLoadFileMessage(f, szFileName, errorCode);							
@@ -1058,4 +1019,3 @@ LRESULT CALLBACK XAP_Win32FrameImpl::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM
 
 	return DefWindowProc(hwnd, iMsg, wParam, lParam);
 }
-
