@@ -22,6 +22,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 
 #include "ut_debugmsg.h"
 #include "ut_growbuf.h"
@@ -45,6 +46,7 @@ enum
 {
 	TT_ABIPREFERENCES,
 	TT_GEOMETRY,
+	TT_LOG,
 	TT_PLUGIN,
 	TT_RECENT,
 	TT_SCHEME,
@@ -56,6 +58,7 @@ static struct xmlToIdMapping s_Tokens[] =
 {
 	{ "AbiPreferences",		TT_ABIPREFERENCES },
 	{ "Geometry",			TT_GEOMETRY },
+	{ "Log",                TT_LOG},
 	{ "Plugin",				TT_PLUGIN },
 	{ "Recent",				TT_RECENT },
 	{ "Scheme",				TT_SCHEME },
@@ -81,13 +84,13 @@ XAP_PrefsScheme::~XAP_PrefsScheme(void)
 	FREEP(m_szName);
 
 	// loop through and free the values
-	UT_Vector * pVec = m_hash.enumerate ();
+	UT_GenericVector<XML_Char*> * pVec = m_hash.enumerate ();
 
 	UT_uint32 cnt = pVec->size();
 
 	for (UT_uint32 i = 0 ; i < cnt; i++)
 	  {
-	    char * val = static_cast<char *>(pVec->getNthItem (i));
+	    char * val = pVec->getNthItem (i);
 	    FREEP(val);
 	  }
 
@@ -108,21 +111,19 @@ bool XAP_PrefsScheme::setSchemeName(const XML_Char * szNewSchemeName)
 bool XAP_PrefsScheme::setValue(const XML_Char * szKey, const XML_Char * szValue)
 {
 	++m_uTick;
-	const void * pEntry = m_hash.pick(szKey);
+	XML_Char * pEntry = m_hash.pick(szKey);
 	if (pEntry)
 	{
-		if (strcmp(static_cast<const char*>(szValue),static_cast<const char *>(pEntry)) == 0)
+		if (strcmp(szValue,pEntry) == 0)
 			return true;				// equal values, no changes required
 		
+		m_hash.set (szKey, UT_strdup (szValue));
 		FREEP(pEntry);
-		m_hash.set (szKey, 
-			    static_cast<void *>(UT_strdup (szValue)));
 	}
 	else
 	{
 		// otherwise, need to add a new entry
-		m_hash.insert(szKey,
-			      static_cast<void *>(UT_strdup(szValue)));
+		m_hash.insert(szKey, UT_strdup(szValue));
 	}
 
 	m_pPrefs->_markPrefChange( szKey );
@@ -137,22 +138,22 @@ bool XAP_PrefsScheme::setValueBool(const XML_Char * szKey, bool bValue)
 
 bool XAP_PrefsScheme::getValue(const XML_Char * szKey, const XML_Char ** pszValue) const
 {
-	const void *pEntry = m_hash.pick(szKey);
+	XML_Char *pEntry = m_hash.pick(szKey);
 	if (!pEntry)
 		return false;
 
 	if (pszValue)
-		*pszValue = static_cast<const XML_Char*>(pEntry);
+		*pszValue = pEntry;
 	return true;
 }
 
 bool XAP_PrefsScheme::getValue(const UT_String &stKey, UT_String &stValue) const
 {
-	const void * pEntry = m_hash.pick(stKey);
+	XML_Char *pEntry = m_hash.pick(stKey);
 	if (!pEntry)
 		return false;
 
-	stValue = static_cast<const char *>(pEntry);
+	stValue = pEntry;
 	return true;
 }
 
@@ -190,20 +191,20 @@ bool XAP_PrefsScheme::getNthValue(UT_uint32 k, const XML_Char ** pszKey, const X
 //
 // Output prefs in alphabetic Order.
 //
-	UT_Vector vecKeys;
-	UT_Vector * vecD = m_hash.keys();
+	UT_GenericVector<const char*> vecKeys;
+	UT_GenericVector<const UT_String*> * vecD = m_hash.keys();
 	UT_uint32 i=0;
 	vecKeys.clear();
 	for(i=0; i< vecD->getItemCount(); i++)
 	{
-		vecKeys.addItem(static_cast<const void *>(static_cast<UT_String *>(vecD->getNthItem(i))->c_str()));
+		vecKeys.addItem(vecD->getNthItem(i)->c_str());
 	}
 	vecKeys.qsort(compareStrings);
 	delete vecD;
 	const char * szKey = NULL;
 	const char * szValue = NULL;
-	szKey = reinterpret_cast<XML_Char *>(vecKeys.getNthItem(k));
-	szValue = static_cast<const XML_Char *>(m_hash.pick(szKey));
+	szKey = vecKeys.getNthItem(k);
+	szValue = m_hash.pick(szKey);
 	if(szValue && *szValue)
 	{
 		*pszKey = szKey;
@@ -284,16 +285,21 @@ const char * XAP_Prefs::getRecent(UT_uint32 k) const
 	
 void XAP_Prefs::addRecent(const char * szRecent)
 {
-	const char * sz;
+	char * sz;
 	bool bFound = false;
 
 	if (m_iMaxRecent == 0)
 		return;		// NOOP
 
+	if(m_bIgnoreThisOne)
+	{
+		m_bIgnoreThisOne = false;
+		return;
+	}
 	// was it already here? 
 	for (UT_uint32 i=0; i<m_vecRecent.getItemCount(); i++)
 	{
-		sz = reinterpret_cast<const char *>(m_vecRecent.getNthItem(i));
+		sz = m_vecRecent.getNthItem(i);
 		if ((sz==szRecent) || !strcmp(sz, szRecent))
 		{
 			// yep, we're gonna move it up
@@ -306,10 +312,10 @@ void XAP_Prefs::addRecent(const char * szRecent)
 	if (!bFound)
 	{
 		// nope.  make a new copy to store
-		UT_cloneString((char *&)sz, szRecent);
+		UT_cloneString(sz, szRecent);
 	}
 
-	m_vecRecent.insertItemAt(const_cast<void *>(static_cast<const void *>(sz)), 0);
+	m_vecRecent.insertItemAt(sz, 0);
 	_pruneRecent();
 }
 
@@ -318,7 +324,7 @@ void XAP_Prefs::removeRecent(UT_uint32 k)
 	UT_ASSERT(k>0);
 	UT_ASSERT(k<=getRecentCount());
 
-	char * sz = static_cast<char *>(m_vecRecent.getNthItem(k-1));
+	char * sz = m_vecRecent.getNthItem(k-1);
 	FREEP(sz);
 
 	m_vecRecent.deleteNthItem(k-1);
@@ -334,7 +340,7 @@ void XAP_Prefs::_pruneRecent(void)
 		// nuke the whole thing
 		for (i = static_cast<signed>(count); i > 0 ; i--)
 		{
-			char * sz = static_cast<char *>(m_vecRecent.getNthItem(i-1));
+			char * sz = m_vecRecent.getNthItem(i-1);
 			FREEP(sz);
 		}
 
@@ -387,6 +393,59 @@ bool XAP_Prefs::getGeometry(UT_sint32 *posx, UT_sint32 *posy, UT_uint32 *width,
 	return true;
 }
 
+void XAP_Prefs::log(const char * where, const char * what, XAPPrefsLog_Level level)
+{
+	UT_return_if_fail(where && what);
+	char b[50];
+
+	time_t t = time(NULL);
+
+	// we are inserting the log entries as comments, so we have to
+	// ensure we have no "--" in there (it anoys the parser)
+	UT_UTF8String sWhere(where);
+	UT_UTF8String sWhat(what);
+	UT_UTF8String sDashdash = "--";
+	UT_UTF8String sDash = "-";
+
+	while(strstr(sWhat.utf8_str(), "--"))
+	{
+		sWhat.escape(sDashdash, sDash);
+	}
+	
+	while(strstr(sWhere.utf8_str(), "--"))
+	{
+		sWhere.escape(sDashdash, sDash);
+	}
+
+	strftime(b, 50, "<!-- [%c] ", localtime(&t));
+
+	UT_UTF8String * s = new UT_UTF8String(b);
+	UT_return_if_fail(s);
+
+	switch(level)
+	{
+		case Warning:
+			*s += "warning: ";
+			break;
+
+		case Error:
+			*s += "error:   ";
+		case Log:
+		default:
+			*s += "message: ";
+	}
+	
+	*s += sWhere;
+	*s += " - ";
+	*s += sWhat;
+	*s += " -->";
+
+	// s->escapeXML(); // since we embed these as comments we do not
+	                   // have to do this
+
+	m_vecLog.addItem(s);
+}
+
 
 /*****************************************************************/
 
@@ -400,7 +459,7 @@ XAP_Prefs::XAP_Prefs(XAP_App * pApp)
 	m_builtinScheme = NULL;
 	m_iMaxRecent = atoi(XAP_PREF_DEFAULT_MaxRecent);
 	m_bInChangeBlock = false;
-
+	m_bIgnoreThisOne = false;
 	memset(&m_geom, 0, sizeof(m_geom));
 
 	// NOTE: since constructors cannot report malloc
@@ -419,15 +478,16 @@ XAP_Prefs::~XAP_Prefs(void)
 	UT_VECTOR_PURGEALL(XAP_PrefsScheme *, m_vecPluginSchemes);
 	UT_VECTOR_FREEALL(char *, m_vecRecent);
 	UT_VECTOR_PURGEALL(tPrefsListenersPair *, m_vecPrefsListeners);
+	UT_VECTOR_PURGEALL(UT_UTF8String *, m_vecLog);
 }
 
 /*****************************************************************/
 
-XAP_PrefsScheme * XAP_Prefs::_getNthScheme(UT_uint32 k, const UT_Vector &vecSchemes) const
+XAP_PrefsScheme * XAP_Prefs::_getNthScheme(UT_uint32 k, const UT_GenericVector<XAP_PrefsScheme *> &vecSchemes) const
 {
 	UT_uint32 kLimit = vecSchemes.getItemCount();
 	if (k < kLimit)
-		return static_cast<XAP_PrefsScheme *>(vecSchemes.getNthItem(k));
+		return vecSchemes.getNthItem(k);
 	else
 		return NULL;
 }
@@ -493,7 +553,6 @@ bool XAP_Prefs::addPluginScheme(XAP_PrefsScheme * pNewScheme)
 	return (m_vecPluginSchemes.addItem(pNewScheme) == 0);
 }
 
-
 XAP_PrefsScheme * XAP_Prefs::getCurrentScheme(bool bCreate)
 {
 	if (bCreate)
@@ -640,7 +699,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 		_startElement_SystemDefaultFile (name, atts);
 		return;
 	}
-
+	UT_DEBUGMSG(("Looking for %s \n",name));
 	XAP_PrefsScheme * pNewScheme = NULL; // must be freed
 	
 	if (!m_parserState.m_parserStatus)		// eat if already had an error
@@ -655,10 +714,16 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 #endif
 									n_compare));
 	if (!id)
+	{
+		UT_DEBUGMSG(("Didin't find it! Abort! \n"));
+		UT_ASSERT(0);
 		return;
-
+	}
 	switch (id->m_type)
 	{
+		case TT_LOG: // ignore
+			break;
+			
 		case TT_ABIPREFERENCES:
 		{
 		m_parserState.m_bFoundAbiPreferences = true;
@@ -874,7 +939,7 @@ void XAP_Prefs::startElement(const XML_Char *name, const XML_Char **atts)
 				UT_XML_cloneString((XML_Char *&)sz, a[1]);
 
 				// NOTE: we keep the copied string in the vector
-				m_vecRecent.addItem(static_cast<void *>(sz));
+				m_vecRecent.addItem(sz);
 			}
 
 			a += 2;
@@ -1090,7 +1155,7 @@ bool XAP_Prefs::savePrefsFile(void)
 	fprintf(fp,"<!-- =====================================================================  -->\n");
 	fprintf(fp,"\n");
 
-#ifdef DEBUG
+#if 1 //def DEBUG -- want this always in, it helps debugging
 	if (XAP_App::s_szBuild_ID && XAP_App::s_szBuild_ID[0])
 	{
 		fprintf(fp,"<!--         Build_ID          = ");
@@ -1303,8 +1368,10 @@ bool XAP_Prefs::savePrefsFile(void)
 		for (k=0; k<kLimit; k++)
 		{
 			const char * szRecent = getRecent(k+1);
+			UT_UTF8String utf8string( szRecent );
+			utf8string.escapeXML();
 
-			fprintf(fp,"\t\tname%d=\"%s\"\n",k+1,szRecent);
+			fprintf(fp,"\t\tname%d=\"%s\"\n",k+1,utf8string.utf8_str());
 		}
 				
 		fprintf(fp,"\t\t/>\n");
@@ -1317,7 +1384,21 @@ bool XAP_Prefs::savePrefsFile(void)
 		fprintf(fp,"\t\tflags=\"%d\"\n", m_geom.m_flags); 
 		fprintf(fp,"\t\t/>\n");
 	}
+	
+	{
+		// now the log
+		fprintf(fp, "\n\t<Log>\n");
 
+		UT_uint32 kLimit = m_vecLog.getItemCount();
+		for (UT_uint32 k = 0; k < kLimit; ++k)
+		{
+			UT_UTF8String * s = (UT_UTF8String *) m_vecLog.getNthItem(k);
+			fprintf(fp,"\t%s\n", s->utf8_str());
+		}
+	
+		fprintf(fp, "\t</Log>\n");
+	}
+	
 	fprintf(fp,"\n</AbiPreferences>\n");
 	
 Cleanup:
@@ -1416,7 +1497,7 @@ void XAP_Prefs::addListener	  ( PrefsListener pFunc, void *data )
 	pPair->m_pFunc = pFunc;
 	pPair->m_pData = data;
 
-	m_vecPrefsListeners.addItem( static_cast<void *>(pPair) );
+	m_vecPrefsListeners.addItem(pPair);
 }
 
 // optional parameter, data.  If given (i.e., != 0), will try to match data,
@@ -1428,7 +1509,7 @@ void XAP_Prefs::removeListener ( PrefsListener pFunc, void *data )
 
 	for ( index = 0; index < m_vecPrefsListeners.getItemCount(); index++ )
 	{
-		pPair = static_cast<tPrefsListenersPair *>(m_vecPrefsListeners.getNthItem(index));
+		pPair = m_vecPrefsListeners.getNthItem(index);
 		UT_ASSERT(pPair);
 		if ( pPair ) {
 			if ( pPair->m_pFunc == pFunc && (!data || pPair->m_pData == data) ) {
@@ -1480,8 +1561,7 @@ void XAP_Prefs::_sendPrefsSignal( UT_StringPtrMap *hash  )
 	UT_uint32	index;
 	for ( index = 0; index < m_vecPrefsListeners.getItemCount(); index++ )
 	{
-		tPrefsListenersPair *p = (tPrefsListenersPair *)
-			m_vecPrefsListeners.getNthItem( index );
+		tPrefsListenersPair *p = m_vecPrefsListeners.getNthItem( index );
 
 		UT_ASSERT(p && p->m_pFunc);
 	
