@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -39,25 +39,108 @@
 #include "px_CR_SpanChange.h"
 #include "px_CR_Strux.h"
 #include "pd_Style.h"
+#include "pp_Revision.h"
 
 #define SETP(p,v)	do { if (p) (*(p)) = (v); } while (0)
 
 /****************************************************************/
 /****************************************************************/
 
+bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
+									 PT_DocPosition dpos1,
+									 PT_DocPosition dpos2,
+									 const XML_Char ** attributes,
+								  const XML_Char ** properties)
+{
+	// if dpos1 == dpos2 we are inserting a fmt mark; this must be chanelled throught
+	// the non-revision branch ...
+	if(m_pDocument->isMarkRevisions() && dpos1 != dpos2)
+	{
+		const XML_Char name[] = "revision";
+		const XML_Char * pRevision = NULL;
+
+		// first retrive the starting and ending fragments
+		pf_Frag * pf1, * pf2;
+		PT_BlockOffset Offset1, Offset2;
+
+		if(!getFragsFromPositions(dpos1,dpos2, &pf1, &Offset1, &pf2, &Offset2))
+			return false;
+
+		// now we have to traverse the fragments and change their
+		// formatting
+
+		pf_Frag * pTemp;
+		pf_Frag * pEnd = pf2->getNext();
+
+		for(pTemp = pf1; pTemp && (pTemp != pEnd); pTemp = pTemp->getNext())
+		{
+			// get attributes for this fragement
+			const PP_AttrProp * pAP;
+			pRevision = NULL;
+			
+			if(_getSpanAttrPropHelper(pTemp, &pAP))
+			{
+				pAP->getAttribute(name, pRevision);
+			}
+			
+			PP_RevisionAttr Revisions(pRevision);
+
+
+			// if the request is for removal of fmt, in the revision mode, we still
+			// have to add these props (the removal is indicated by their emptiness)
+			// as we cannot rely on callers to set these correctly, we have to emtpy
+			// them ourselves
+			const XML_Char ** attrs = attributes;
+			const XML_Char ** props = properties;
+							
+			if(ptc == PTC_RemoveFmt)
+			{
+				attrs = UT_setPropsToNothing(attributes);
+				props = UT_setPropsToNothing(properties);
+			}
+			
+			Revisions.addRevision(m_pDocument->getRevisionId(),PP_REVISION_FMT_CHANGE,attrs,props);
+
+			if(attrs != attributes)
+				delete[] attrs;
+							
+			if(props != properties)
+				delete[] props;
+			
+			const XML_Char * ppRevAttrib[3];
+			ppRevAttrib[0] = name;
+			ppRevAttrib[1] = Revisions.getXMLstring();
+			ppRevAttrib[2] = NULL;
+
+			PT_DocPosition dposEnd = UT_MIN(dpos2,dpos1 + pTemp->getLength());
+
+			if(!_realChangeSpanFmt(PTC_AddFmt, dpos1, dposEnd, ppRevAttrib,NULL))
+				return false;
+
+			dpos1 = dposEnd;
+		}
+
+		return true;
+	}
+	else
+	{
+		return _realChangeSpanFmt(ptc, dpos1, dpos2, attributes, properties);
+	}
+}
+
 bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_uint32 length,
 									  PT_AttrPropIndex indexNewAP,
 									  pf_Frag ** ppfNewEnd, UT_uint32 * pfragOffsetNewEnd)
 {
-	UT_ASSERT(length > 0);
-	UT_ASSERT(fragOffset+length <= pft->getLength());
-	
+	UT_return_val_if_fail (length > 0,false);
+	UT_return_val_if_fail (fragOffset+length <= pft->getLength(), false);
+
 	// insert a format change within this text fragment.
 
 	// TODO for each place in this function where we apply a change
 	// TODO see if the new fragment could be coalesced with something
 	// TODO already in the fragment list.
-	
+
 	if ((fragOffset == 0) && (length == pft->getLength()))
 	{
 		// we have an exact match (we are changing the entire fragment).
@@ -66,7 +149,7 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 		// first we try the pft->next -- if that works, we can stop because
 		// the unlink will take care of checking pft->next with pft->prev.
 		// if it doesn't work, we then try pft->prev.
-		
+
 		pf_Frag * pfNext = pft->getNext();
 		if (pfNext && pfNext->getType()==pf_Frag::PFT_Text)
 		{
@@ -112,11 +195,11 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 		}
 
 		// otherwise, we just overwrite the indexAP on this fragment.
-		
+
 		pft->setIndexAP(indexNewAP);
 		SETP(ppfNewEnd, pft->getNext());
 		SETP(pfragOffsetNewEnd, 0);
-		
+
 		return true;
 	}
 
@@ -148,13 +231,13 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 				pft->adjustOffsetLength(bi_2,len_2);
 				SETP(ppfNewEnd, pft);
 				SETP(pfragOffsetNewEnd, 0);
-		
+
 				return true;
 			}
 		}
 
 		// otherwise, we need to actually split this one....
-		
+
 		pf_Frag_Text * pftNew = new pf_Frag_Text(this,bi_1,len_1,indexNewAP,pft->getField());
 		if (!pftNew)
 			return false;
@@ -164,7 +247,7 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 
 		SETP(ppfNewEnd, pft);
 		SETP(pfragOffsetNewEnd, 0);
-		
+
 		return true;
 	}
 
@@ -209,7 +292,7 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 
 		SETP(ppfNewEnd, pftNew->getNext());
 		SETP(pfragOffsetNewEnd, 0);
-		
+
 		return true;
 	}
 
@@ -223,9 +306,9 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 	PT_BufIndex bi_2 = m_varset.getBufIndex(pft->getBufIndex(),fragOffset);
 	PT_BufIndex bi_3 = m_varset.getBufIndex(pft->getBufIndex(),fragOffset+length);
 	pf_Frag_Text * pft_2 = new pf_Frag_Text(this,bi_2,len_2,indexNewAP,pft->getField());
-	UT_ASSERT(pft_2);
+	UT_return_val_if_fail (pft_2, false);
 	pf_Frag_Text * pft_3 = new pf_Frag_Text(this,bi_3,len_3,pft->getIndexAP(),pft->getField());
-	UT_ASSERT(pft_3);
+	UT_return_val_if_fail (pft_3, false);
 
 	pft->changeLength(len_1);
 	m_fragments.insertFrag(pft,pft_2);
@@ -233,10 +316,10 @@ bool pt_PieceTable::_fmtChangeSpan(pf_Frag_Text * pft, UT_uint32 fragOffset, UT_
 
 	SETP(ppfNewEnd, pft_3);
 	SETP(pfragOffsetNewEnd, 0);
-		
+
 	return true;
 }
-	
+
 bool pt_PieceTable::_fmtChangeSpanWithNotify(PTChangeFmt ptc,
 												pf_Frag_Text * pft, UT_uint32 fragOffset,
 												PT_DocPosition dpos,
@@ -257,14 +340,14 @@ bool pt_PieceTable::_fmtChangeSpanWithNotify(PTChangeFmt ptc,
 		return true;
 	}
 
-	UT_ASSERT(fragOffset+length <= pft->getLength());
-	
+	UT_return_val_if_fail (fragOffset+length <= pft->getLength(), false);
+
 	PT_AttrPropIndex indexNewAP;
 	PT_AttrPropIndex indexOldAP = pft->getIndexAP();
 	bool bMerged;
 
 	bMerged = m_varset.mergeAP(ptc,indexOldAP,attributes,properties,&indexNewAP,getDocument());
-	UT_ASSERT(bMerged);
+	UT_ASSERT_HARMLESS(bMerged);
 
 	if (indexOldAP == indexNewAP)		// the requested change will have no effect on this fragment.
 	{
@@ -278,10 +361,10 @@ bool pt_PieceTable::_fmtChangeSpanWithNotify(PTChangeFmt ptc,
 			SETP(ppfNewEnd, pft);
 			SETP(pfragOffsetNewEnd, fragOffset+length);
 		}
-		
+
 		return true;
 	}
-	
+
 	// we do this before the actual change because various fields that
 	// we need may be blown away during the change.  we then notify all
 	// listeners of the change.
@@ -293,7 +376,7 @@ bool pt_PieceTable::_fmtChangeSpanWithNotify(PTChangeFmt ptc,
 										 dpos, indexOldAP,indexNewAP,
 										 m_varset.getBufIndex(pft->getBufIndex(),fragOffset),
 										 length,blockOffset);
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr,false);
 	bool bResult = _fmtChangeSpan(pft,fragOffset,length,indexNewAP,ppfNewEnd,pfragOffsetNewEnd);
 
 	// add record to history.  we do not attempt to coalesce these.
@@ -303,17 +386,27 @@ bool pt_PieceTable::_fmtChangeSpanWithNotify(PTChangeFmt ptc,
 	return bResult;
 }
 
-bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
+bool pt_PieceTable::_realChangeSpanFmt(PTChangeFmt ptc,
 									 PT_DocPosition dpos1,
 									 PT_DocPosition dpos2,
 									 const XML_Char ** attributes,
 									 const XML_Char ** properties)
 {
 	// apply a span-level formatting change to the given region.
-	
-	UT_ASSERT(m_pts==PTS_Editing);
-    _tweakFieldSpan(dpos1,dpos2);
 
+	UT_return_val_if_fail (m_pts==PTS_Editing,false);
+    _tweakFieldSpan(dpos1,dpos2);
+//
+// Deal with case of exactly selecting the endOfFootnote
+//
+	pf_Frag * pfEndDum = m_fragments.findFirstFragBeforePos(dpos2);
+	if(isEndFootnote(pfEndDum))
+	{
+		if(dpos2 > dpos1)
+		{
+			dpos2--;
+		}
+	}
 //
 // Deal with addStyle
 //
@@ -326,14 +419,14 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 // OK for styles we expand out all defined properties including BasedOn styles
 // Then we use these to eliminate any specfic properties in the current strux
 // Then properties in the current strux will resolve to those defined in the
-// style (they exist there) to specifc values in strux (if not overridden by 
+// style (they exist there) to specifc values in strux (if not overridden by
 // the style) then finally to default value.
 //
 		const XML_Char * szStyle = UT_getAttribute(PT_STYLE_ATTRIBUTE_NAME,attributes);
 		PD_Style * pStyle = NULL;
-		UT_ASSERT(szStyle);
+		UT_return_val_if_fail (szStyle,false);
 		getDocument()->getStyle(szStyle,&pStyle);
-		UT_ASSERT(pStyle);
+		UT_return_val_if_fail (pStyle,false);
 		UT_Vector vProps;
 //
 // Get the vector of properties
@@ -373,13 +466,13 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 		}
 		return bRes;
 	}
-	
-	UT_ASSERT(dpos1 < dpos2);
+
+	UT_return_val_if_fail (dpos1 < dpos2,false);
 	bool bHaveAttributes, bHaveProperties;
 	bHaveAttributes = (attributes && *attributes);
 	bHaveProperties = (lProps && *lProps);
-	UT_ASSERT(bHaveAttributes || bHaveProperties); // must have something to do
-    
+	UT_return_val_if_fail (bHaveAttributes || bHaveProperties, false); // must have something to do
+
 	pf_Frag * pf_First;
 	pf_Frag * pf_End;
 	PT_BlockOffset fragOffset_First;
@@ -387,7 +480,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 
 	bool bFound;
 	bFound = getFragsFromPositions(dpos1,dpos2,&pf_First,&fragOffset_First,&pf_End,&fragOffset_End);
-	UT_ASSERT(bFound);
+	UT_return_val_if_fail (bFound, false);
 
 #if 0
 	{
@@ -396,11 +489,11 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 
 		bool bFound1 = getFragFromPosition(dpos1,&pf1,&fo1);
 		bool bFound2 = getFragFromPosition(dpos2,&pf2,&fo2);
-		UT_ASSERT(bFound1 && bFound2);
-		UT_ASSERT((pf1==pf_First) && (fragOffset_First==fo1));
-		UT_ASSERT((pf2==pf_End) && (fragOffset_End==fo2));
+		UT_return_val_if_fail (bFound1 && bFound2, false);
+		UT_return_val_if_fail ((pf1==pf_First) && (fragOffset_First==fo1), false);
+		UT_return_val_if_fail ((pf2==pf_End) && (fragOffset_End==fo2), false);
 	}
-#endif	
+#endif
 
 	// see if the amount of text to be changed is completely
 	// contained within a single fragment.  if so, we have a
@@ -425,32 +518,32 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 	UT_uint32 fragOffsetNewEnd;
 
 	UT_uint32 length = dpos2 - dpos1;
-	while (length >= 0)
+	while (length != 0)
 	{
 		// FIXME: Special check to support a FmtMark at the end of the
 		// document. This is necessary because FmtMarks don't have a
 		// length...  See bug 452.
-		if (0 == length 
+		if (0 == length
 			&& (!pf_First || pf_Frag::PFT_FmtMark != pf_First->getType()))
 			break;
 
-		UT_ASSERT(dpos1+length==dpos2);
+		UT_return_val_if_fail (dpos1+length==dpos2, false);
 
 		UT_uint32 lengthInFrag = pf_First->getLength() - fragOffset_First;
 		UT_uint32 lengthThisStep = UT_MIN(lengthInFrag, length);
-		
+
 		switch (pf_First->getType())
 		{
 		case pf_Frag::PFT_EndOfDoc:
 		default:
 			UT_DEBUGMSG(("fragment type: %d\n",pf_First->getType()));
-			UT_ASSERT(0);
+			UT_ASSERT_HARMLESS(0);
 			if(bApplyStyle)
 			{
 				FREEP(sProps);
 			}
 			return false;
-			
+
 		case pf_Frag::PFT_Strux:
 			{
 				// we are only applying span-level changes, so we ignore strux.
@@ -459,6 +552,12 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 				pfNewEnd = pf_First->getNext();
 				fragOffsetNewEnd = 0;
 				pfsContainer = static_cast<pf_Frag_Strux *> (pf_First);
+				bool bFoundStrux = false;
+				if(isEndFootnote(pfsContainer))
+				{
+					bFoundStrux = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+					UT_return_val_if_fail (bFoundStrux, false);
+				}
 			}
 			break;
 
@@ -468,7 +567,12 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 				{
 					bool bFoundStrux;
 					bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-					UT_ASSERT(bFoundStrux);
+					UT_return_val_if_fail (bFoundStrux,false);
+					if(isEndFootnote(pfsContainer))
+					{
+						bFoundStrux = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+						UT_return_val_if_fail (bFoundStrux,false);
+					}
 				}
 
 				bool bResult;
@@ -476,7 +580,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 											   fragOffset_First,dpos1,lengthThisStep,
 											   attributes,lProps,
 											   pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-				UT_ASSERT(bResult);
+				UT_return_val_if_fail (bResult,false);
 			}
 			break;
 
@@ -486,7 +590,12 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 				{
 					bool bFoundStrux;
 					bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-					UT_ASSERT(bFoundStrux);
+					UT_return_val_if_fail (bFoundStrux,false);
+					if(isEndFootnote(pfsContainer))
+					{
+						bFoundStrux = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+						UT_return_val_if_fail (bFoundStrux,false);
+					}
 				}
 
 				bool bResult;
@@ -494,7 +603,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 												 fragOffset_First,dpos1,lengthThisStep,
 												 attributes,lProps,
 												 pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-				UT_ASSERT(bResult);
+				UT_return_val_if_fail (bResult,false);
 			}
 			break;
 
@@ -504,14 +613,20 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 				{
 					bool bFoundStrux;
 					bFoundStrux = _getStruxFromPosition(dpos1,&pfsContainer);
-					UT_ASSERT(bFoundStrux);
+					UT_return_val_if_fail (bFoundStrux,false);
+					if(isEndFootnote(pfsContainer))
+					{
+						bFoundStrux = _getStruxFromFragSkip(pfsContainer,&pfsContainer);
+						UT_return_val_if_fail (bFoundStrux,false);
+					}
+
 				}
 
 				bool bResult;
 				bResult = _fmtChangeFmtMarkWithNotify(ptc,static_cast<pf_Frag_FmtMark *>(pf_First),
 												  dpos1, attributes,lProps,
 												  pfsContainer,&pfNewEnd,&fragOffsetNewEnd);
-				UT_ASSERT(bResult);
+				UT_return_val_if_fail (bResult,false);
 			}
 			break;
 
@@ -519,7 +634,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 
 		dpos1 += lengthThisStep;
 		length -= lengthThisStep;
-		
+
 		// since _fmtChange{Span,FmtMark,...}WithNotify(), can delete pf_First, mess with the
 		// fragment list, and does some aggressive coalescing of
 		// fragments, we cannot just do a pf_First->getNext() here.
@@ -538,7 +653,7 @@ bool pt_PieceTable::changeSpanFmt(PTChangeFmt ptc,
 
 	if (!bSimple)
 		endMultiStepGlob();
-		
+
 	return true;
 }
 
