@@ -48,8 +48,13 @@
 /*****************************************************************/
 
 pt_PieceTable::pt_PieceTable(PD_Document * pDocument)
-  : m_pts(PTS_Create), m_hashStyles(11), m_pDocument(pDocument),
-    m_atomicGlobCount(0)
+  : m_pts(PTS_Create), 
+	m_history(this),
+	m_hashStyles(11), 
+	m_pDocument(pDocument),
+    m_atomicGlobCount(0),
+	m_bDoingTheDo(false),
+	m_bDoNotTweakPosition(false)
 {
 
 	setPieceTableState(PTS_Create);
@@ -58,12 +63,12 @@ pt_PieceTable::pt_PieceTable(PD_Document * pDocument)
 
 pt_PieceTable::~pt_PieceTable()
 {
-	UT_HASH_PURGEDATA(PD_Style *, &m_hashStyles, delete);
+	m_hashStyles.purgeData();
 }
 
 void pt_PieceTable::setPieceTableState(PTState pts)
 {
-	UT_ASSERT(pts >= m_pts);
+	UT_return_if_fail (pts >= m_pts);
 
 	if ((m_pts==PTS_Create) && (pts==PTS_Loading))
 	{
@@ -122,7 +127,7 @@ bool pt_PieceTable::insertStruxNoUpdateBefore(PL_StruxDocHandle sdh, PTStruxType
 	{		
 		PT_AttrPropIndex pAPIold = indexAP;
 		bool bMerged = m_varset.mergeAP(PTC_AddFmt,pAPIold,attributes,NULL,&indexAP,getDocument());
-		UT_ASSERT(bMerged);
+		UT_ASSERT_HARMLESS(bMerged);
 	}
 //
 // create a strux
@@ -133,11 +138,8 @@ bool pt_PieceTable::insertStruxNoUpdateBefore(PL_StruxDocHandle sdh, PTStruxType
 // Insert it.
 //
 	pf_Frag * pfPrev = pfs->getPrev();
-	UT_ASSERT(pfPrev);
-	if(!pfPrev)
-	{
-		return false;
-	}
+	UT_return_val_if_fail (pfPrev,false);
+
 	m_fragments.insertFrag(pfPrev,static_cast<pf_Frag *>(pNewStrux));
 #if 0
 	m_pDocument->miniDump(sdh,8);
@@ -195,7 +197,7 @@ bool pt_PieceTable::_struxHasContent(pf_Frag_Strux * pfs) const
 
 bool pt_PieceTable::getAttrProp(PT_AttrPropIndex indexAP, const PP_AttrProp ** ppAP) const
 {
-	UT_ASSERT(ppAP);
+	UT_return_val_if_fail (ppAP,false);
 
 	const PP_AttrProp * pAP = m_varset.getAP(indexAP);
 	if (!pAP)
@@ -207,31 +209,25 @@ bool pt_PieceTable::getAttrProp(PT_AttrPropIndex indexAP, const PP_AttrProp ** p
 
 bool pt_PieceTable::_getSpanAttrPropHelper(pf_Frag * pf, const PP_AttrProp ** ppAP) const
 {
-#define ReturnThis(type,pf)													\
-	do {	type * _pf_ = static_cast< type *>((pf));						\
-			const PP_AttrProp * pAP = m_varset.getAP(_pf_->getIndexAP());	\
-			*ppAP = pAP;													\
-			return (pAP != NULL);											\
-	} while (0)
-
 	switch (pf->getType())
 	{
 	case pf_Frag::PFT_FmtMark:
-		ReturnThis( pf_Frag_FmtMark, pf );
-
 	case pf_Frag::PFT_Text:
-		ReturnThis( pf_Frag_Text, pf );
-
 	case pf_Frag::PFT_Object:
-		ReturnThis( pf_Frag_Object, pf );
+		{	
+			const PP_AttrProp * pAP = m_varset.getAP(pf->getIndexAP());
+		    *ppAP = pAP;
+		    return true;
+		}
 
 	case pf_Frag::PFT_Strux:
 	case pf_Frag::PFT_EndOfDoc:
 	default:
-		*ppAP = NULL;
-		return false;
+		{
+			*ppAP = NULL;
+		}
 	}
-#undef ReturnThis
+	return false;
 }
 
 
@@ -242,13 +238,13 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 	// offset zero now refers to the first character in the block, so adding
 	// fl_BLOCK_STRUX_OFFSET to the offset in the call is no longer necessary.
 
-	UT_ASSERT(sdh);
-	UT_ASSERT(ppAP);
+	UT_return_val_if_fail (sdh,false);
+	UT_return_val_if_fail (ppAP,false);
 
 	const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
-	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
+	UT_return_val_if_fail (pf->getType() == pf_Frag::PFT_Strux,false);
 	const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
-	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
+	UT_return_val_if_fail (pfsBlock->getStruxType() == PTX_Block,false);
 
 	UT_uint32 cumOffset = 0;
 	UT_uint32 cumEndOffset = 0;
@@ -269,7 +265,7 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 			return _getSpanAttrPropHelper(pfTemp,ppAP);
 		}
 
-		UT_ASSERT(offset > cumOffset);
+		UT_return_val_if_fail (offset > cumOffset,false);
 
 		if (offset == cumEndOffset)		// there's a frag boundary exactly where we want. pfTemp is to our left.
 		{
@@ -284,7 +280,7 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 			return _getSpanAttrPropHelper(pfTemp,ppAP);
 		}
 
-		UT_ASSERT(offset < cumEndOffset);
+		UT_return_val_if_fail (offset < cumEndOffset,false);
 
 		// the place we want is inside of a fragment, so just use it.
 		return _getSpanAttrPropHelper(pfTemp,ppAP);
@@ -294,7 +290,9 @@ bool pt_PieceTable::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, boo
 	return false;
 }
 
-
+#if 0
+// I will leave the code here for now to aid in debugging any problems
+// with the new iterator (should there be any, that is) Tomas, Nov 15, 2003
 bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 								  const UT_UCSChar ** ppSpan, UT_uint32 * pLength) const
 {
@@ -305,9 +303,9 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 	*pLength = 0;
 
 	const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
-	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
+	UT_return_val_if_fail (pf->getType() == pf_Frag::PFT_Strux,false);
 	const pf_Frag_Strux * pfsBlock = static_cast<const pf_Frag_Strux *> (pf);
-	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
+	UT_return_val_if_fail (pfsBlock->getStruxType() == PTX_Block,false);
 	xxx_UT_DEBUGMSG(("getSpanPtr: Requested offset %d \n",offset));
 	
 	UT_uint32 cumOffset = pf->getLength();
@@ -326,7 +324,7 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 			if (pfTemp->getType() != pf_Frag::PFT_Text)
 			{
 				xxx_UT_DEBUGMSG(("getSpanPtr: Error 1 offset %d cumOffset %d \n",offset,cumOffset));
-//				UT_ASSERT(0);
+//				UT_ASSERT_HARMLESS(0);
 				return false;
 			}
 
@@ -360,7 +358,7 @@ bool pt_PieceTable::getSpanPtr(PL_StruxDocHandle sdh, UT_uint32 offset,
 	xxx_UT_DEBUGMSG(("getSpanPtr: Error 3 offset %d cumOffset %d \n",offset,cumOffset));
 	return false;
 }
-
+#endif
 
 PD_Document * pt_PieceTable::getDocument(void)
 {
@@ -379,7 +377,7 @@ PD_Document * pt_PieceTable::getDocument(void)
 bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
                                    UT_GrowBuf * pgb) const
 {
-    UT_ASSERT(pgb);
+    UT_return_val_if_fail (pgb,false);
 
     const pf_Frag * pf = static_cast<const pf_Frag *>(sdh);
     UT_return_val_if_fail(pf->getType() == pf_Frag::PFT_Strux, false);
@@ -395,7 +393,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
         switch (pfTemp->getType())
         {
         default:
-            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
         case pf_Frag::PFT_Strux:
 //
 // Have to deal with embedded section struxes like Footnotes. We do this by
@@ -406,11 +404,21 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
 		{
 			UT_GrowBufElement valz = 0;
             bool bAppended;
+			//
+			// Deal with TOC's. Since TOC's are always placed right before
+			// the end of the paragraph we terminate the fill here.
+			//
+			pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pfTemp);
+			if(pfs->getStruxType() == PTX_SectionTOC)
+			{
+				pfTemp = NULL;
+				break;
+			}
 			if(isFootnote(pfTemp))
 			{
 				countFoots++;
 				bAppended = pgb->ins(bufferOffset,&valz,1);
-				UT_ASSERT(bAppended);
+				UT_return_val_if_fail (bAppended,false);
 				bufferOffset++;
 				pfTemp = pfTemp->getNext();
 				break;
@@ -424,7 +432,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
 					break;
 				}
 				bAppended = pgb->ins(bufferOffset,&valz,1);
-				UT_ASSERT(bAppended);
+				UT_return_val_if_fail (bAppended,false);
 				bufferOffset++;
 				pfTemp = pfTemp->getNext();
 				break;
@@ -432,7 +440,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
 			if(countFoots>0)
 			{
 				bAppended = pgb->ins(bufferOffset,&valz,1);
-				UT_ASSERT(bAppended);
+				UT_return_val_if_fail (bAppended,false);
 				bufferOffset++;
 				pfTemp = pfTemp->getNext();
 				break;
@@ -456,7 +464,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
 
             bool bAppended;
             bAppended = pgb->ins(bufferOffset,reinterpret_cast<const UT_GrowBufElement*>(pSpan),length);
-            UT_ASSERT(bAppended);
+            UT_return_val_if_fail (bAppended,false);
 
             bufferOffset += length;
         }
@@ -500,7 +508,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
             bool bAppended;
             bAppended = pgb->ins(bufferOffset, reinterpret_cast<UT_GrowBufElement*>(pSpaces), length);
             delete[] pSpaces;
-            UT_ASSERT(bAppended);
+            UT_return_val_if_fail (bAppended,false);
 
             bufferOffset += length;
         }
@@ -509,7 +517,7 @@ bool pt_PieceTable::getBlockBuf(PL_StruxDocHandle sdh,
         }
     }
 
-    UT_ASSERT(bufferOffset == pgb->getLength());
+    UT_return_val_if_fail (bufferOffset == pgb->getLength(),false);
     return true;
 }
 
@@ -562,6 +570,7 @@ PT_DocPosition pt_PieceTable::getFragPosition(const pf_Frag * pfToFind) const
 	return  pfToFind->getPos();
 }
 
+
 bool pt_PieceTable::getFragFromPosition(PT_DocPosition docPos,
 										   pf_Frag ** ppf,
 										   PT_BlockOffset * pFragOffset) const
@@ -587,8 +596,8 @@ bool pt_PieceTable::getFragFromPosition(PT_DocPosition docPos,
 		xxx_UT_DEBUGMSG(("SEVIOR: I found frag %d \n",getFragNumber(pfLast)));
 	}
 
-	UT_ASSERT(pfLast);
-	UT_ASSERT(pfLast->getType() == pf_Frag::PFT_EndOfDoc);
+	UT_return_val_if_fail (pfLast,false);
+	UT_return_val_if_fail (pfLast->getType() == pf_Frag::PFT_EndOfDoc,false);
 
 	return true;
 }
@@ -642,9 +651,9 @@ bool pt_PieceTable::getFragsFromPositions(PT_DocPosition dPos1, PT_DocPosition d
 {
 	// compute the (fragment,offset) pairs for each position given.
 
-	UT_ASSERT(dPos1 <= dPos2);
-	UT_ASSERT(ppf1);
-	UT_ASSERT(pOffset1);
+	UT_return_val_if_fail (dPos1 <= dPos2,false);
+	UT_return_val_if_fail (ppf1,false);
+	UT_return_val_if_fail (pOffset1,false);
 
 	// the first set has to be done the hard way.
 
@@ -669,7 +678,7 @@ bool pt_PieceTable::getFragsFromPositions(PT_DocPosition dPos1, PT_DocPosition d
 
 	// a FmtMark has length zero.  we don't want to find it here.
 	// rather we want the thing to the right of it.
-	UT_ASSERT(pf->getType() != pf_Frag::PFT_FmtMark);
+	UT_return_val_if_fail (pf->getType() != pf_Frag::PFT_FmtMark,false);
 
 	if (ppf2)
 		*ppf2 = pf;
@@ -729,7 +738,7 @@ bool pt_PieceTable::isEndFootnote(pf_Frag * pf) const
 	if(pf->getType() == pf_Frag::PFT_Strux)
 	{
 		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
-		if((pfs->getStruxType() == PTX_EndFootnote) || (pfs->getStruxType() == PTX_EndEndnote))
+		if((pfs->getStruxType() == PTX_EndFootnote) || (pfs->getStruxType() == PTX_EndEndnote) || (pfs->getStruxType() == PTX_EndTOC))
 		{
 			return true;
 		}
@@ -743,7 +752,7 @@ bool pt_PieceTable::isFootnote(pf_Frag * pf) const
 	if(pf->getType() == pf_Frag::PFT_Strux)
 	{
 		pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux *>(pf);
-		if((pfs->getStruxType() == PTX_SectionFootnote) || (pfs->getStruxType() == PTX_SectionEndnote))
+		if((pfs->getStruxType() == PTX_SectionFootnote) || (pfs->getStruxType() == PTX_SectionEndnote) || (pfs->getStruxType() == PTX_SectionTOC))
 		{
 			return true;
 		}
@@ -758,11 +767,13 @@ bool pt_PieceTable::_getStruxFromPosition(PT_DocPosition docPos,
 	// return the strux fragment immediately prior (containing)
 	// the given absolute document position.  If bSkip set, skip past
     // Footnote struxes.
+	xxx_UT_DEBUGMSG(("_getStruxFromPosition bSkipFootnotes %d initial pos %d \n",bSkipFootnotes,docPos));
 	UT_sint32 countEndFootnotes = 0;
 	pf_Frag * pfFirst = m_fragments.findFirstFragBeforePos( docPos);
-
+	xxx_UT_DEBUGMSG(("Frag found %x pos %d \n",pfFirst,pfFirst->getPos()));
 	if(isEndFootnote(pfFirst))
 		countEndFootnotes++;
+	xxx_UT_DEBUGMSG(("Frag found, count endfootnotes %d  \n",countEndFootnotes));
 
 	while(pfFirst && pfFirst->getPrev() && pfFirst->getPos() >= docPos)
 	{
@@ -779,6 +790,7 @@ bool pt_PieceTable::_getStruxFromPosition(PT_DocPosition docPos,
 		   (bSkipFootnotes && ((countEndFootnotes > 0) || isFootnote(pfFirst) || isEndFootnote(pfFirst)))))
 	{
 		pfFirst = pfFirst->getPrev();
+		xxx_UT_DEBUGMSG(("Frag found %x pos %d \n",pfFirst,pfFirst->getPos()));
 		if(pfFirst == NULL)
 		{
 			break;
@@ -801,16 +813,18 @@ bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
 {
 	// return the strux fragment of the given type containing
 	// the given absolute document position.
-	UT_ASSERT(ppfs);
+	UT_return_val_if_fail (ppfs, false);
 	xxx_UT_DEBUGMSG(("_getStruxOfTypeFromPosition: looking for type %d \n",pts));
 	*ppfs = NULL;
 
 	pf_Frag_Strux * pfs = NULL;
-	if (!_getStruxFromPosition(dpos,&pfs, !(pts == PTX_EndFootnote || pts == PTX_SectionFootnote)))
+	bool wantFootNoteType = (pts == PTX_EndFootnote || pts == PTX_SectionFootnote || pts == PTX_EndEndnote || pts == PTX_SectionEndnote || pts == PTX_SectionTOC || pts == PTX_EndTOC);
+	if (!_getStruxFromPosition(dpos,&pfs, !wantFootNoteType))
 		return false;
 
 	PTStruxType pfsType = pfs->getStruxType();
-	if (pfsType == pts || (pts == PTX_Section && pfsType == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsType == PTX_SectionFootnote) || (pts == PTX_SectionEndnote && pfsType == PTX_SectionEndnote) || (pts == PTX_SectionTable && pfsType == PTX_SectionTable) || (pts == PTX_SectionCell && pfsType == PTX_SectionCell) || (pts == PTX_EndTable && pfsType == PTX_EndTable) || (pts == PTX_EndCell && pfsType == PTX_EndCell)  )		// is it of the type we want
+	xxx_UT_DEBUGMSG(("Got strux type %d \n",pfs->getStruxType()));
+	if (pfsType == pts || (pts == PTX_Section && pfsType == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsType == PTX_SectionFootnote) || (pts == PTX_SectionEndnote && pfsType == PTX_SectionEndnote) || (pts == PTX_SectionTable && pfsType == PTX_SectionTable) || (pts == PTX_SectionCell && pfsType == PTX_SectionCell) || (pts == PTX_EndTable && pfsType == PTX_EndTable) || (pts == PTX_EndCell && pfsType == PTX_EndCell) || (pts == PTX_SectionTOC && pfsType == PTX_SectionTOC)  )		// is it of the type we want
 	{
 		*ppfs = pfs;
 		return true;
@@ -821,7 +835,15 @@ bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
 	for (pf_Frag * pf=pfs; (pf); pf=pf->getPrev())
 		if (pf->getType() == pf_Frag::PFT_Strux)
 		{
-			pf_Frag_Strux * pfsTemp = static_cast<pf_Frag_Strux *>(pf);
+			pf_Frag_Strux * pfsTemp = NULL;
+			if(!wantFootNoteType && isEndFootnote(pf))
+			{
+				_getStruxFromFragSkip(pf,&pfsTemp);
+			}
+			else
+			{
+				pfsTemp = static_cast<pf_Frag_Strux *>(pf);
+			}
 			if(pfsTemp->getStruxType() == PTX_EndTable)
 			{
 				numEndTable++;
@@ -830,7 +852,7 @@ bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
 			{
 				numEndTable--;
 			}
-			if (pfsTemp->getStruxType() == pts || (pts == PTX_Section && pfsTemp->getStruxType() == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsTemp->getStruxType() == PTX_SectionFootnote) || (pts == PTX_EndFootnote && pfsTemp->getStruxType() == PTX_EndFootnote) || (pts == PTX_SectionEndnote && pfsTemp->getStruxType() == PTX_SectionEndnote) || (pts == PTX_EndEndnote && pfsTemp->getStruxType() == PTX_EndEndnote))	// did we find it
+			if (pfsTemp->getStruxType() == pts || (pts == PTX_Section && pfsTemp->getStruxType() == PTX_SectionHdrFtr) || (pts == PTX_SectionFootnote && pfsTemp->getStruxType() == PTX_SectionFootnote) || (pts == PTX_EndFootnote && pfsTemp->getStruxType() == PTX_EndFootnote) || (pts == PTX_SectionEndnote && pfsTemp->getStruxType() == PTX_SectionEndnote) || (pts == PTX_EndEndnote && pfsTemp->getStruxType() == PTX_EndEndnote) || (pts == PTX_SectionTOC && pfsTemp->getStruxType() == PTX_SectionTOC) || (pts == PTX_EndTOC && pfsTemp->getStruxType() == PTX_EndTOC))	// did we find it
 			{
 				if(((numEndTable < 0) && (pfsTemp->getStruxType()==PTX_SectionTable)) || (numEndTable == 0 && (pfsTemp->getStruxType()!=PTX_SectionTable)))
 				{
@@ -853,6 +875,123 @@ bool pt_PieceTable::_getStruxOfTypeFromPosition(PT_DocPosition dpos,
 	return false;
 }
 
+/*!
+ * Scan backwards from the given frag until a start hyperlink is found.
+ * This method is used to determine if a frag is inside a hyperlink span.
+ * Returns NULL if:
+ * (a) It encounters a strux first.
+ * (b) It encounters an end hyperlink first
+ * (c) It encounters the begin of document
+ */
+pf_Frag *    pt_PieceTable::_findPrevHyperlink(pf_Frag * pfStart)
+{
+	pf_Frag * pf = pfStart;
+	pf_Frag_Object *pOb = NULL;
+	UT_sint32 iCountFootnotes = 0;
+	while(pf)
+	{
+		if(pf->getType() == pf_Frag::PFT_Strux)
+		{
+			if(isEndFootnote(pf))
+			{
+				iCountFootnotes++;
+			}
+			else if(isFootnote(pf))
+			{
+				iCountFootnotes--;
+			}
+			else if(iCountFootnotes == 0)
+			{
+				return NULL;
+			}
+		}
+		if(pf->getType() == pf_Frag::PFT_Object)
+		{
+			pOb = static_cast<pf_Frag_Object*>(pf);
+			if(pOb->getObjectType() == PTO_Hyperlink)
+			{
+				const PP_AttrProp * pAP = NULL;
+				pOb->getPieceTable()->getAttrProp(pOb->getIndexAP(),&pAP);
+				UT_return_val_if_fail (pAP, NULL);
+				const XML_Char* pszHref = NULL;
+				const XML_Char* pszHname  = NULL;
+				UT_uint32 k = 0;
+				while((pAP)->getNthAttribute(k++,pszHname, pszHref))
+				{
+					if(!UT_strcmp(pszHname, "xlink:href"))
+				    {
+						return pf;
+					}
+				}
+				return NULL;
+			}
+
+		}
+		pf = pf->getPrev();
+	}
+	return NULL;
+}
+
+
+/*!
+ * Scan backwards fromthe given frag until an end hyperlink is found.
+ * This method is used to determine if a frag is inside a hyperlink span.
+ * Returns NULL if:
+ * (a) It encounters a strux first.
+ * (b) It encounters a start hyperlink first
+ * (c) It encounters the end of document
+ */
+pf_Frag *    pt_PieceTable::_findNextHyperlink(pf_Frag * pfStart)
+{
+	pf_Frag * pf = pfStart;
+	pf_Frag_Object *pOb = NULL;
+	UT_sint32 iCountFootnotes = 0;
+	while(pf && pf != m_fragments.getLast())
+	{
+		if(pf->getType() == pf_Frag::PFT_Strux)
+		{
+			if(isFootnote(pf))
+			{
+				iCountFootnotes++;
+			}
+			else if(isEndFootnote(pf))
+			{
+				iCountFootnotes--;
+			}
+			else if(iCountFootnotes == 0)
+			{
+				return NULL;
+			}
+		}
+		if(pf->getType() == pf_Frag::PFT_Object)
+		{
+			pOb = static_cast<pf_Frag_Object*>(pf);
+			if(pOb->getObjectType() == PTO_Hyperlink)
+			{
+				const PP_AttrProp * pAP = NULL;
+				pOb->getPieceTable()->getAttrProp(pOb->getIndexAP(),&pAP);
+				UT_return_val_if_fail (pAP, NULL);
+				const XML_Char* pszHref = NULL;
+				const XML_Char* pszHname  = NULL;
+				UT_uint32 k = 0;
+				while((pAP)->getNthAttribute(k++,pszHname, pszHref))
+				{
+					if(!UT_strcmp(pszHname, "xlink:href"))
+				    {
+						return NULL;
+					}
+				}
+				//
+				// No start marker => Must be end marker - GOT IT!
+				//
+				return pf;
+			}
+		}
+		pf = pf->getNext();
+	}
+	return NULL;
+}
+
 bool pt_PieceTable::_getStruxFromFrag(pf_Frag * pfStart, pf_Frag_Strux ** ppfs) const
 {
 	// return the strux frag immediately prior to (containing)
@@ -861,7 +1000,6 @@ bool pt_PieceTable::_getStruxFromFrag(pf_Frag * pfStart, pf_Frag_Strux ** ppfs)
 	*ppfs = NULL;
 
 	pf_Frag * pf;
-
 	for (pf=pfStart->getPrev(); (pf && (pf->getType() != pf_Frag::PFT_Strux)); pf=pf->getPrev())
 		;
 	if (!pf)
@@ -872,6 +1010,46 @@ bool pt_PieceTable::_getStruxFromFrag(pf_Frag * pfStart, pf_Frag_Strux ** ppfs)
 }
 
 
+bool pt_PieceTable::_getNextStruxAfterFragSkip(pf_Frag *pfStart, pf_Frag_Strux ** ppfs)
+{
+
+	*ppfs = NULL;
+
+	pf_Frag * pf;
+	UT_sint32 countFoots = 0;
+	if(isFootnote(pfStart))
+	{
+		countFoots++;
+	}
+	pf = pfStart->getNext();
+	if(pf && isFootnote(pf))
+	{
+		countFoots++;
+	}
+	xxx_UT_DEBUGMSG(("_getStruxFromFragStrux: 1 countFoots %d \n",countFoots));
+	while(pf && (pf->getType() != pf_Frag::PFT_EndOfDoc) && ((pf->getType() != pf_Frag::PFT_Strux) || (countFoots > 0) 
+				 || isFootnote(pf) || isEndFootnote(pf)))
+	{
+		pf=pf->getNext();
+		if(isFootnote(pf))
+		{
+			countFoots++;
+		}
+		else if(isEndFootnote(pf))
+		{
+			countFoots--;
+		}
+		xxx_UT_DEBUGMSG(("_getStruxFromFragStrux: 2 countFoots %d \n",countFoots));
+	}
+		;
+	if (!pf)
+		return false;
+
+	*ppfs = static_cast<pf_Frag_Strux *>(pf);
+	return true;
+}
+
+
 bool pt_PieceTable::_getStruxFromFragSkip(pf_Frag * pfStart, pf_Frag_Strux ** ppfs) const
 {
 	// return the strux frag immediately prior to (containing)
@@ -890,16 +1068,20 @@ bool pt_PieceTable::_getStruxFromFragSkip(pf_Frag * pfStart, pf_Frag_Strux ** pp
 	{
 		countFoots++;
 	}
+	if(isFootnote(pf))
+	{
+		countFoots--;
+	}
 	xxx_UT_DEBUGMSG(("_getStruxFromFragStrux: 1 countFoots %d \n",countFoots));
 	while(pf && ((pf->getType() != pf_Frag::PFT_Strux) || (countFoots > 0) 
 				 || isFootnote(pf) || isEndFootnote(pf)))
 	{
 		pf=pf->getPrev();
-		if(isFootnote(pf))
+		if(pf && isFootnote(pf))
 		{
 			countFoots--;
 		}
-		else if(isEndFootnote(pf))
+		else if(pf && isEndFootnote(pf))
 		{
 			countFoots++;
 		}
@@ -920,8 +1102,12 @@ UT_uint32 pt_PieceTable::_computeBlockOffset(pf_Frag_Strux * pfs,pf_Frag * pfTar
 	UT_uint32 sum;
 	pf_Frag * pf;
 
-	for (pf=pfs->getNext(), sum=0; (pf!=pfTarget); sum+=pf->getLength(), pf=pf->getNext())
+	for (pf=pfs->getNext(), sum=0; (pf && (pf!=pfTarget)); sum+=pf->getLength(), pf=pf->getNext())
 		;
+	if(pf == NULL)
+	{
+		return 0;
+	}
 
 	return sum;
 }
@@ -941,7 +1127,7 @@ bool pt_PieceTable::_changePointWithNotify(PT_DocPosition dpos)
 	PX_ChangeRecord * pcr
 		= new PX_ChangeRecord(PX_ChangeRecord::PXT_ChangePoint,
 							  dpos, 0);
-	UT_ASSERT(pcr);
+	UT_return_val_if_fail (pcr,false);
 
 	m_history.addChangeRecord(pcr);
 	m_pDocument->notifyListeners(NULL, pcr);
