@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -53,9 +53,18 @@
 #include "ap_BeOSDialog_Insert_DateTime.h"
 #include "ap_BeOSDialog_Field.h"
 #include "ap_BeOSDialog_Tab.h"
-#include "ap_BeOSDialog_Columns.h" 
+#include "ap_BeOSDialog_Columns.h"
 #include "ap_BeOSDialog_PageNumbers.h"
-
+#include "ap_BeOSDialog_New.h"
+#include "ap_BeOSDialog_HdrFtr.h"
+#include "ap_BeOSDialog_InsertBookmark.h"
+#include "ap_BeOSDialog_InsertHyperlink.h"
+#include "ap_BeOSDialog_Background.h"
+#include "ap_BeOSDialog_MarkRevisions.h"
+#include "ap_BeOSDialog_ListRevisions.h"
+#if 0
+#include "ap_BeOSDialog_Download_File.h"
+#endif
 	// ... add new dialogs here ...
 
 #else
@@ -68,15 +77,15 @@
 	DeclareDialog(XAP_DIALOG_ID_FONT,			XAP_BeOSDialog_FontChooser)
 	DeclareDialog(XAP_DIALOG_ID_LANGUAGE,			XAP_BeOSDialog_Language)
 	DeclareDialog(XAP_DIALOG_ID_ENCODING,			XAP_BeOSDialog_Encoding)
-	DeclareDialog(XAP_DIALOG_ID_WINDOWMORE,		XAP_BeOSDialog_WindowMore) 
-	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_BeOSDialog_Zoom) 
-	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_BeOSDialog_About) 
-	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,		XAP_BeOSDialog_Insert_Symbol) 
+	DeclareDialog(XAP_DIALOG_ID_WINDOWMORE,		XAP_BeOSDialog_WindowMore)
+	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_BeOSDialog_Zoom)
+	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_BeOSDialog_About)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,		XAP_BeOSDialog_Insert_Symbol)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE,	XAP_BeOSDialog_FileOpenSaveAs)
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_BeOSDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_BeOSDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_BeOSDialog_Break)
-    DeclareDialog(AP_DIALOG_ID_GOTO,            AP_BeOSDialog_Goto)
+	DeclareDialog(AP_DIALOG_ID_GOTO,            AP_BeOSDialog_Goto)
 	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_BeOSDialog_Spell)
 	DeclareDialog(AP_DIALOG_ID_STYLES,			AP_BeOSDialog_Styles)
 	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_BeOSDialog_Paragraph)
@@ -89,6 +98,17 @@
 	DeclareDialog(AP_DIALOG_ID_PAGE_NUMBERS,		AP_BeOSDialog_PageNumbers)
         DeclareDialog(XAP_DIALOG_ID_FILE_IMPORT, XAP_BeOSDialog_FileOpenSaveAs)
         DeclareDialog(XAP_DIALOG_ID_FILE_EXPORT, XAP_BeOSDialog_FileOpenSaveAs)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_FILE, XAP_BeOSDialog_FileOpenSaveAs)
+ 	DeclareDialog(AP_DIALOG_ID_INSERTBOOKMARK,	AP_BeOSDialog_InsertBookmark)
+ 	DeclareDialog(AP_DIALOG_ID_INSERTHYPERLINK,	AP_BeOSDialog_InsertHyperlink)
+ 	DeclareDialog(AP_DIALOG_ID_HDRFTR,          AP_BeOSDialog_HdrFtr)
+ 	DeclareDialog(AP_DIALOG_ID_FILE_NEW, AP_BeOSDialog_New)
+ 	DeclareDialog(AP_DIALOG_ID_BACKGROUND,		AP_BeOSDialog_Background)
+ 	DeclareDialog(AP_DIALOG_ID_MARK_REVISIONS,	AP_BeOSDialog_MarkRevisions)
+ 	DeclareDialog(AP_DIALOG_ID_LIST_REVISIONS,	AP_BeOSDialog_ListRevisions)
+#if 0
+	DeclareDialog(AP_DIALOG_ID_DOWNLOAD_FILE,	AP_BeOSDialog_Download_File)
+#endif
 	// ... also add new dialogs here ...
 
 #endif
