@@ -35,7 +35,13 @@
 #include <string.h>
 #include <io.h>
 #include <fcntl.h>
+
+#if !defined(__WINE__) && (!defined(_MSC_VER) || _MSC_VER < 1310)
 #include <iostream.h>
+#elif _MSC_VER >= 1310
+#include <iostream>
+#endif
+
 #include <ole2.h>
 
 #include "ut_debugmsg.h"
@@ -87,6 +93,8 @@
 #include "ap_Strings.h"
 
 #include "pt_PieceTable.h"
+
+#include "gr_Painter.h"
 // extern prototype - this is defined in ap_EditMethods.cpp
 extern XAP_Dialog_MessageBox::tAnswer s_CouldNotLoadFileMessage(XAP_Frame * pFrame, const char * pNewFile, UT_Error errorCode);
 /*****************************************************************/
@@ -133,31 +141,86 @@ bool AP_Win32App::initialize(void)
 	bool bSuccess = true;
 	const char * szUserPrivateDirectory = getUserPrivateDirectory();
 	bool bVerified = s_createDirectoryIfNecessary(szUserPrivateDirectory);
-	UT_ASSERT(bVerified);
+	UT_return_val_if_fail (bVerified, false);
 
 	// load the preferences.
 	
 	m_prefs = new AP_Win32Prefs(this);
+	UT_return_val_if_fail (m_prefs, false);
+	
 	m_prefs->fullInit();
 		   
 	// now that preferences are established, let the xap init
 
 	m_pClipboard = new AP_Win32Clipboard();
-	UT_ASSERT(m_pClipboard);
+	UT_return_val_if_fail (m_pClipboard, false);
 	   
 	m_pEMC = AP_GetEditMethods();
-	UT_ASSERT(m_pEMC);
+	UT_return_val_if_fail (m_pEMC, false);
 
 	m_pBindingSet = new AP_BindingSet(m_pEMC);
-	UT_ASSERT(m_pBindingSet);
+	UT_return_val_if_fail (m_pBindingSet, false);
 	
 	m_pMenuActionSet = AP_CreateMenuActionSet();
-	UT_ASSERT(m_pMenuActionSet);
+	UT_return_val_if_fail (m_pMenuActionSet,false);
 
 	m_pToolbarActionSet = AP_CreateToolbarActionSet();
-	UT_ASSERT(m_pToolbarActionSet);
+	UT_return_val_if_fail (m_pToolbarActionSet,false);
 
-	if (! XAP_Win32App::initialize())
+	//////////////////////////////////////////////////////////////////
+	// load the dialog and message box strings
+	//////////////////////////////////////////////////////////////////
+	
+	{
+		// assume we will be using the builtin set (either as the main
+		// set or as the fallback set).
+		
+		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
+		UT_return_val_if_fail (pBuiltinStringSet, false);
+		m_pStringSet = pBuiltinStringSet;
+
+		// see if we should load an alternate set from the disk
+		
+		const char * szDirectory = NULL;
+		const char * szStringSet = NULL;
+
+		if (   (getPrefsValue(AP_PREF_KEY_StringSet,&szStringSet))
+			&& (szStringSet)
+			&& (*szStringSet)
+			&& (UT_stricmp(szStringSet,AP_PREF_DEFAULT_StringSet) != 0))
+		{
+			getPrefsValueDirectory(true,AP_PREF_KEY_StringSetDirectory,&szDirectory);
+			UT_return_val_if_fail ((szDirectory) && (*szDirectory), false);
+
+			char * szPathname = (char *)calloc(sizeof(char),strlen(szDirectory)+strlen(szStringSet)+100);
+			UT_return_val_if_fail (szPathname, false);
+
+			sprintf(szPathname,"%s%s%s.strings",
+					szDirectory,
+					((szDirectory[strlen(szDirectory)-1]=='\\') ? "" : "\\"),
+					szStringSet);
+
+			AP_DiskStringSet * pDiskStringSet = new AP_DiskStringSet(this);
+			UT_return_val_if_fail (pDiskStringSet, false);
+
+			if (pDiskStringSet->loadStringsFromDisk(szPathname))
+			{
+				pDiskStringSet->setFallbackStringSet(m_pStringSet);
+				m_pStringSet = pDiskStringSet;
+				UT_DEBUGMSG(("Using StringSet [%s]\n",szPathname));
+			}
+			else
+			{
+				UT_DEBUGMSG(("Unable to load StringSet [%s] -- using builtin strings instead.\n",szPathname));				
+				DELETEP(pDiskStringSet);
+			}
+				
+			free(szPathname);
+		}
+	}
+
+	// AP_App::initilize() calls for us XAP_Win32App::initialize()
+	if (! AP_App::initialize())
 		return false;
 
 	
@@ -182,57 +245,7 @@ bool AP_Win32App::initialize(void)
 		SpellManager::instance();
 	}
 	
-	//////////////////////////////////////////////////////////////////
-	// load the dialog and message box strings
-	//////////////////////////////////////////////////////////////////
 	
-	{
-		// assume we will be using the builtin set (either as the main
-		// set or as the fallback set).
-		
-		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
-		UT_ASSERT(pBuiltinStringSet);
-		m_pStringSet = pBuiltinStringSet;
-
-		// see if we should load an alternate set from the disk
-		
-		const char * szDirectory = NULL;
-		const char * szStringSet = NULL;
-
-		if (   (getPrefsValue(AP_PREF_KEY_StringSet,&szStringSet))
-			&& (szStringSet)
-			&& (*szStringSet)
-			&& (UT_stricmp(szStringSet,AP_PREF_DEFAULT_StringSet) != 0))
-		{
-			getPrefsValueDirectory(true,AP_PREF_KEY_StringSetDirectory,&szDirectory);
-			UT_ASSERT((szDirectory) && (*szDirectory));
-
-			char * szPathname = (char *)calloc(sizeof(char),strlen(szDirectory)+strlen(szStringSet)+100);
-			UT_ASSERT(szPathname);
-
-			sprintf(szPathname,"%s%s%s.strings",
-					szDirectory,
-					((szDirectory[strlen(szDirectory)-1]=='\\') ? "" : "\\"),
-					szStringSet);
-
-			AP_DiskStringSet * pDiskStringSet = new AP_DiskStringSet(this);
-			UT_ASSERT(pDiskStringSet);
-
-			if (pDiskStringSet->loadStringsFromDisk(szPathname))
-			{
-				pDiskStringSet->setFallbackStringSet(m_pStringSet);
-				m_pStringSet = pDiskStringSet;
-				UT_DEBUGMSG(("Using StringSet [%s]\n",szPathname));
-			}
-			else
-			{
-				UT_DEBUGMSG(("Unable to load StringSet [%s] -- using builtin strings instead.\n",szPathname));				
-				DELETEP(pDiskStringSet);
-			}
-				
-			free(szPathname);
-		}
-	}
 	// Now we have the strings loaded we can populate the field names correctly
 	int i;
 	
@@ -289,7 +302,7 @@ bool AP_Win32App::initialize(void)
 
 	// Ensure that common control DLL is loaded
 	HINSTANCE hinstCC = LoadLibrary("comctl32.dll");
-	UT_ASSERT(hinstCC);
+	UT_return_val_if_fail (hinstCC, false);
 	InitCommonControlsEx_fn  pInitCommonControlsEx = NULL;
 	if( hinstCC != NULL )
 		pInitCommonControlsEx = (InitCommonControlsEx_fn)GetProcAddress( hinstCC, "InitCommonControlsEx" );
@@ -381,6 +394,9 @@ bool AP_Win32App::shutdown(void)
 	if (m_prefs->getAutoSavePrefs())
 		m_prefs->savePrefsFile();
 
+	delete m_prefs;
+	m_prefs = NULL;
+
 	return true;
 }
 
@@ -403,7 +419,7 @@ bool AP_Win32App::getPrefsValueDirectory(bool bAppSpecific,
 	const XML_Char * dir = ((bAppSpecific) ? getAbiSuiteAppDir() : getAbiSuiteLibDir());
 
 	static XML_Char buf[1024];
-	UT_ASSERT((strlen(dir) + strlen(psz) + 2) < sizeof(buf));
+	UT_return_val_if_fail ((strlen(dir) + strlen(psz) + 2) < sizeof(buf), false);
 	
 	sprintf(buf,"%s\\%s",dir,psz);
 	*pszValue = buf;
@@ -415,7 +431,7 @@ const char * AP_Win32App::getAbiSuiteAppDir(void) const
 	// we return a static string, use it quickly.
 	
 	static XML_Char buf[1024];
-	UT_ASSERT((strlen(getAbiSuiteLibDir()) + strlen(ABIWORD_APP_LIBDIR) + 2) < sizeof(buf));
+	UT_return_val_if_fail ((strlen(getAbiSuiteLibDir()) + strlen(ABIWORD_APP_LIBDIR) + 2) < sizeof(buf), NULL);
 
 	sprintf(buf,"%s\\%s",getAbiSuiteLibDir(),ABIWORD_APP_LIBDIR);
 	return buf;
@@ -959,7 +975,7 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
  		pDocLy =	pFrameData->m_pDocLayout;	
  		pView =  pDocLy->getView();		
  				
- 		errorCode = pView->cmdInsertGraphic(pFG, szFile);	  		  		
+ 		errorCode = pView->cmdInsertGraphic(pFG);	  		  		
  	
 		DELETEP(pIEG);
  		//DELETEP(pFG);		
@@ -1000,6 +1016,9 @@ bool AP_Win32App::_pasteFormatFromClipboard(PD_DocumentRange * pDocRange, const
 
 bool AP_Win32App::canPasteFromClipboard(void)
 {
+	if (!getLastFocussedFrame()) 
+		return false;
+
 	AP_Win32FrameImpl * pFrameImp = static_cast<AP_Win32FrameImpl*>(getLastFocussedFrame()->getFrameImpl());
 	UT_return_val_if_fail(pFrameImp, false);
 	
@@ -1100,8 +1119,15 @@ static LRESULT CALLBACK _SplashWndProc(HWND hWnd, UINT message, WPARAM wParam, L
         hdc = BeginPaint(hWnd, &ps);
 		{
 			// TODO: find XAP_App pointer for this
-			GR_Graphics * pG = new GR_Win32Graphics(hdc, hwndSplash, 0);
-			pG->drawImage(pSplash, 0, 0);
+			GR_Win32AllocInfo ai(hdc, hwndSplash,0);
+			
+			GR_Graphics * pG = XAP_App::getApp()->newGraphics(ai);
+			{
+				GR_Painter GP(pG);
+
+				GP.drawImage(pSplash, 0, 0);
+			}
+
 			DELETEP(pG);
 		}
         EndPaint(hWnd, &ps);
@@ -1151,7 +1177,7 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		wndclass.hIconSm       = LoadIcon(hInstance, MAKEINTRESOURCE(AP_RID_ICON_APPLICATION_16)) /* app->getSmallIcon() */;
 
 		a = RegisterClassEx(&wndclass);
-		UT_ASSERT(a);
+		UT_ASSERT_HARMLESS(a);
 
 		// get the extents of the desktop window
 		RECT rect;
@@ -1165,18 +1191,21 @@ static GR_Image * _showSplash(HINSTANCE hInstance, const char * szAppName)
 		// create a centered window the size of our bitmap
 		hwndSplash = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST,s_SplashWndClassName, 
 								  NULL, WS_POPUP | WS_BORDER,
-								  (rect.right  / 2) - (iSplashWidth  / 2),
-								  (rect.bottom / 2) - (iSplashHeight / 2),
-								  iSplashWidth,
-								  iSplashHeight,
+								  (rect.right  / 2) - (iSplashWidth  / 2) - 1, // subtract 1 pixel to account for WS_BORDER
+								  (rect.bottom / 2) - (iSplashHeight / 2) - 1, // subtract 1 pixel to account for WS_BORDER
+								  iSplashWidth + 2, // must add 2 to window size, WS_BORDER takes up one pixel on each side
+								  iSplashHeight + 2, // must add 2 to window size, WS_BORDER takes up one pixel on each side
 								  NULL, NULL, hInstance, NULL);
-		UT_ASSERT(hwndSplash);
+		UT_ASSERT_HARMLESS(hwndSplash);
     
 		if (hwndSplash) 
 		{
 			// create image first
 			// TODO: find XAP_App pointer for this
-			GR_Graphics * pG = new GR_Win32Graphics(GetDC(hwndSplash), hwndSplash, 0);
+			GR_Win32AllocInfo ai(GetDC(hwndSplash), hwndSplash,0);
+			
+			GR_Graphics * pG = XAP_App::getApp()->newGraphics(ai);
+			
 			pSplash = pG->createNewImage("splash", pBB, pG->tlu(iSplashWidth), pG->tlu(iSplashHeight));
 			DELETEP(pG);
 			
@@ -1218,7 +1247,7 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 	HINSTANCE hinstRich = LoadLibrary("riched32.dll");
 	if (!hinstRich)
 		hinstRich = LoadLibrary("riched20.dll");
-	UT_ASSERT(hinstRich);
+	UT_return_val_if_fail (hinstRich, 1);
 	
 	AP_Win32App * pMyWin32App;
 
@@ -1248,7 +1277,7 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 
 	// Consider the user saved preferences for the Splash Screen
    	const XAP_Prefs * pPrefs = pMyWin32App->getPrefs();
-	UT_ASSERT(pPrefs);
+	UT_ASSERT_HARMLESS(pPrefs);
     if (pPrefs && pPrefs->getPrefsValueBool (AP_PREF_KEY_ShowSplash, &bSplashPref))
 	{
 		bShowSplash = bShowSplash && bSplashPref;
@@ -1291,9 +1320,7 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 //	
 
 
-#if  !defined(__MINGW32__)  && !defined(DEBUG) 
-__try
-#endif	
+try
 {		
 	UT_uint32 iHeight = 0, iWidth = 0, t_flag =0;
 	UT_sint32 iPosX = 0, iPosY = 0;
@@ -1311,7 +1338,7 @@ __try
 		for(UT_uint32 i = 0;i<pMyWin32App->m_vecFrames.getItemCount();i++)
 		{
 			AP_Win32Frame * curFrame = (AP_Win32Frame*)pMyWin32App->m_vecFrames[i];
-			UT_ASSERT(curFrame);
+			UT_return_val_if_fail (curFrame, 1);
 		
 			HWND hwnd = curFrame->getTopLevelWindow();
 			ShowWindow(hwnd, iCmdShow);
@@ -1353,31 +1380,61 @@ __try
 	delete pMyWin32App;
 	
 	
-}// end of thes block is controlled by the SEH 
+}// end of thes block is controlled by the Exception Handler
 
 //
 // If an exception happens, with "catch" the block
 // and then the save it into disk
 //
-#if  !defined(__MINGW32__)  && !defined(DEBUG) 
-__except (1)
+catch (...)
 {
+#ifdef DEBUG
+	throw;
+#endif
+
 	AP_Win32App *pApp = (AP_Win32App *) XAP_App::getApp();
 	
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp,1);
+
+	// first of all, try to save the current prefs (so that any log entries are dumped
+	// onto disk -- this allows us to save useful info for dbg purposes) we will enclose
+	// this inside of a try/catch block, so that in the (unlikely) case something goes
+	// badly wrong when writing the prefs file, we still get chance to save the open
+	// documents
+
+	try
+	{
+		if(pApp->getPrefs())
+		{
+			pApp->getPrefs()->savePrefsFile();
+		}
+	}
+	catch(...)
+	{
+		// do nothing
+	}
 	
 	UT_uint32 i = 0;
 	
+	IEFileType abiType = IE_Imp::fileTypeForSuffix("abw");
 	for(;i<pApp->m_vecFrames.getItemCount();i++)
 	{
 		AP_Win32Frame * curFrame = (AP_Win32Frame*)pApp->m_vecFrames[i];
-		UT_ASSERT(curFrame);
-		
-		if (NULL == curFrame->getFilename())
-		  curFrame->backup(".abw.saved");
-		else
-		  curFrame->backup(".saved");
+		UT_return_val_if_fail (curFrame,1);
 
+		// again, we want to catch any exception thrown while saving individual documents,
+		// in order to run through the whole loop
+		try
+		{
+			if (NULL == curFrame->getFilename())
+				curFrame->backup(".abw.saved", abiType);
+			else
+				curFrame->backup(".saved", abiType);
+		}
+		catch(...)
+		{
+			// do nothing
+		}
 	}	
 
 	// Tell the user was has just happened
@@ -1387,10 +1444,7 @@ __except (1)
 		curFrame->showMessageBox(AP_STRING_ID_MSG_Exception,XAP_Dialog_MessageBox::b_O, XAP_Dialog_MessageBox::a_OK);
 		
 	}
-		
-
 }// end of except
-#endif
 
 	SET_CRT_DEBUG_FIELD( _CRTDBG_LEAK_CHECK_DF );
 	return msg.wParam;
@@ -1506,22 +1560,34 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args)
 	else
 	if (Args->m_sPrintTo) 
 	{
-		#if 0
 		if ((Args->m_sFile = poptGetArg (Args->poptcon)) != NULL)
 		{
 			UT_DEBUGMSG(("DOM: Printing file %s\n", Args->m_sFile));
+			AP_Convert conv ;
+
+			if (Args->m_sMerge)
+				conv.setMergeSource (Args->m_sMerge);
+
+			if (Args->m_impProps)
+				conv.setImpProps (Args->m_impProps);
+			if (Args->m_expProps)
+				conv.setExpProps (Args->m_expProps);
 			
-			AP_Convert * conv = new AP_Convert(pMyWin32App);
-			conv->setVerbose(Args->m_iVerbose);
+			UT_String s = "AbiWord: ";
+			s+= Args->m_sFile;
 			
-			PS_Graphics * pG = new PS_Graphics ((Args->m_sPrintTo[0] == '|' ? Args->m_sPrintTo+1 : Args->m_sPrintTo), Args->m_sFile, 
-												pMyUnixApp->getApplicationName(), pMyUnixApp->getFontManager(),
-												(Args->m_sPrintTo[0] != '|'), pMyUnixApp);
+			GR_Graphics * pG = GR_Win32Graphics::getPrinterGraphics(Args->m_sPrintTo, s.c_str());
+			if(!pG)
+			{
+				// do not assert here, if the graphics creation failed, the static
+				// constructor has asserted already somewhere more relevant
+				return false;
+			}
 			
-			conv->print (Args->m_sFile, pG);
+			conv.setVerbose(Args->m_iVerbose);
+			conv.print (Args->m_sFile, pG);
 	      
 			delete pG;
-			delete conv;
 		}
 		else
 		{
@@ -1529,9 +1595,7 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args)
 			printf("Error: no file to print!\n");
 		}
 
-		if (!Args->m_iShow)
-			return false;
-		#endif
+		return false;
 	}
 
 	if(Args->m_sPlugin)
@@ -1545,7 +1609,7 @@ bool AP_Win32App::doWindowlessArgs(const AP_Args *Args)
 		bool bFound = false;
 		if(Args->m_sPlugin != NULL)
 		{
-			const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+			const UT_GenericVector<class XAP_Module *> *pVec = XAP_ModuleManager::instance().enumModules ();
 			for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
 			{
 				pModule = (XAP_Module *)pVec->getNthItem (i);
@@ -1690,5 +1754,3 @@ UT_UTF8String	AP_Win32App::s_fromAnsiToUTF8(const char* szIn)
 
 	return sRslt;
 }
-
-
