@@ -1,19 +1,19 @@
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
@@ -27,7 +27,7 @@
 
 #include "xap_App.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Win32FrameImpl.h"
 #include "fv_View.h"
 
 #include "ap_Strings.h"
@@ -40,10 +40,10 @@
 #include "ap_Win32Resources.rc2"
 #include "xap_Win32Toolbar_Icons.h"
 #include "xap_Win32Dlg_FontChooser.h"
+#include "xap_Win32DialogHelper.h"
+#include "ap_Win32App.h"
 
 #ifdef _MSC_VER
-// Workaround for Microsofts inability to follow international standards.
-#define for if (0) {} else for
 // MSVC++ warns about using 'this' in initializer list.
 #pragma warning(disable: 4355)
 #endif
@@ -61,6 +61,8 @@ AP_Win32Dialog_Lists::static_constructor(	XAP_DialogFactory* pFactory,
 AP_Win32Dialog_Lists::AP_Win32Dialog_Lists(	XAP_DialogFactory* pDlgFactory,
 											XAP_Dialog_Id id)
 :	AP_Dialog_Lists(pDlgFactory,id),
+	m_bDestroy_says_stopupdating(false),
+	m_bAutoUpdate_happening_now(false),
 	m_pAutoUpdateLists(0),
 	_win32Dialog(this),
 	m_pPreviewWidget(0),
@@ -76,6 +78,7 @@ AP_Win32Dialog_Lists::~AP_Win32Dialog_Lists(void)
 		m_pAutoUpdateLists->stop();
 		DELETEP(m_pAutoUpdateLists);
 	}
+	DELETEP(m_pPreviewWidget);
 }
 
 /*****************************************************************/
@@ -104,6 +107,8 @@ void AP_Win32Dialog_Lists::runModeless(XAP_Frame * pFrame)
 
 BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
+	register int i;
+
 	m_hThisDlg = hWnd;
 	setbisCustomized(false);
 	clearDirty();
@@ -121,7 +126,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 		AP_RID_DIALOG_LIST_SPIN_INDENT_ALIGN	// non-UDS_SETBUDDYINT
 	};
 
-	for (int i = 0; i < NrElements(rgSpinIDs); ++i)
+	for (i = 0; i < NrElements(rgSpinIDs); ++i)
 	{
 		HWND hWndSpin = GetDlgItem(hWnd, rgSpinIDs[i]);
 		if (hWndSpin)
@@ -133,10 +138,12 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 			SendMessage(hWndSpin, UDM_SETRANGE, 0L, range);
 		}
 	}
+	
+	XAP_Win32DialogHelper::s_centerDialog(hWnd);			
 
 	const XAP_StringSet* pSS = m_pApp->getStringSet();
 
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_val_if_fail (pSS,1);	// TODO: Would an error handler be more appropriate here?
 
 	// Set the locale specific strings
 	struct control_id_string_id {
@@ -151,7 +158,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 		AP_RID_DIALOG_LIST_STATIC_FORMAT		, AP_STRING_ID_DLG_Lists_Format,
 		AP_RID_DIALOG_LIST_STATIC_FONT			, AP_STRING_ID_DLG_Lists_Font,
 		AP_RID_DIALOG_LIST_BTN_FONT				, AP_STRING_ID_DLG_Lists_ButtonFont,
-		AP_RID_DIALOG_LIST_STATIC_FORMAT		, AP_STRING_ID_DLG_Lists_DelimiterString,
+		AP_RID_DIALOG_LIST_STATIC_DECIMAL		, AP_STRING_ID_DLG_Lists_DelimiterString,
 		AP_RID_DIALOG_LIST_STATIC_LEVEL			, AP_STRING_ID_DLG_Lists_Level,
 		AP_RID_DIALOG_LIST_STATIC_START_AT		, AP_STRING_ID_DLG_Lists_Start,
 		AP_RID_DIALOG_LIST_STATIC_LIST_ALIGN	, AP_STRING_ID_DLG_Lists_Align,
@@ -162,7 +169,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 		AP_RID_DIALOG_LIST_RADIO_RESUME_PREV_LIST, AP_STRING_ID_DLG_Lists_Resume
 	};
 
-	for (int i = 0; i < NrElements(rgMapping); ++i)
+	for (i = 0; i < NrElements(rgMapping); ++i)
 	{
 		_win32Dialog.setControlText(rgMapping[i].controlId,
 									pSS->getValue(rgMapping[i].stringId));
@@ -189,7 +196,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	HWND hwndChild = GetDlgItem(hWnd, AP_RID_DIALOG_LIST_FRAME_PREVIEW);
 
 	// If we don't get the preview control things are bad!
-	UT_ASSERT(hwndChild);
+	UT_return_val_if_fail (hwndChild,1);
 
 	m_pPreviewWidget = new XAP_Win32PreviewWidget(static_cast<XAP_Win32App *>(m_pApp),
 													  hwndChild,
@@ -209,10 +216,24 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	m_pAutoUpdateLists = UT_Timer::static_constructor(autoupdateLists, this);
 	m_bDestroy_says_stopupdating = false;
 	m_pAutoUpdateLists->set(500);	// auto-updater at 1/2 Hz
+	
+	
 
 	return 1;							// 1 == we did not call SetFocus()
 }
 
+
+void AP_Win32Dialog_Lists::setFoldLevelInGUI(void)
+{
+	UT_ASSERT_HARMLESS(0);
+}
+
+bool AP_Win32Dialog_Lists::isPageLists(void)
+{
+	UT_ASSERT_HARMLESS(0);
+	return true;
+}
+
 BOOL AP_Win32Dialog_Lists::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
 	const WORD wNotifyCode	= HIWORD(wParam);
@@ -368,7 +389,7 @@ void AP_Win32Dialog_Lists::destroy(void)
 	m_bDestroy_says_stopupdating = true;
 	while (m_bAutoUpdate_happening_now == true) ;
 	m_pAutoUpdateLists->stop();
-	setAnswer(AP_Dialog_Lists::a_CLOSE);	
+	setAnswer(AP_Dialog_Lists::a_CLOSE);
 	if (IsWindow(m_hThisDlg))
 	{
 		_win32Dialog.destroyWindow();
@@ -411,22 +432,20 @@ void AP_Win32Dialog_Lists::activate(void)
 
 	iResult = _win32Dialog.bringWindowToTop();
 
-	UT_ASSERT((iResult != 0));
+	UT_ASSERT_HARMLESS((iResult != 0));
 
 	_enableControls();
 }
 
 void AP_Win32Dialog_Lists::notifyActiveFrame(XAP_Frame *pFrame)
 {
-	XAP_Win32Frame* pWin32Frame = static_cast<XAP_Win32Frame *>(pFrame);
-
-	if (_win32Dialog.isParentFrame(*pWin32Frame))
+	if (_win32Dialog.isParentFrame(*pFrame))
 	{
 		return;
 	}
 
 	// change parent
-	_win32Dialog.setParentFrame(pWin32Frame);
+	_win32Dialog.setParentFrame(pFrame);
 	_win32Dialog.bringWindowToTop();
 	_updateCaption();
 
@@ -439,21 +458,6 @@ void AP_Win32Dialog_Lists::notifyActiveFrame(XAP_Frame *pFrame)
 	_previewExposed();
 }
 
-void AP_Win32Dialog_Lists::notifyCloseFrame(XAP_Frame *pFrame)
-{
-	// Check that our hWnd still is valid. This is not always the case,
-	// such as when shutting down the application.
-	if (IsWindow(m_hThisDlg))
-	{
-		XAP_Win32Frame* pWin32Frame = static_cast<XAP_Win32Frame *>(pFrame);
-		if (_win32Dialog.isParentFrame(*pWin32Frame))
-		{
-			setbisCustomized(false);
-			setActiveFrame(getActiveFrame());
-		}
-		_updateCaption();
-	}
-}
 
 void AP_Win32Dialog_Lists::_enableControls(void)
 {
@@ -497,7 +501,7 @@ void AP_Win32Dialog_Lists::_onApply()
 	const bool bResumeList     = _isResumeListChecked();
 
 	// There can be only one.
-	UT_ASSERT((int(bStartNewList) + int(bApplyToCurrent) + int(bResumeList)) == 1);
+	UT_return_if_fail ((int(bStartNewList) + int(bApplyToCurrent) + int(bResumeList)) == 1);
 
 	setbStartNewList(bStartNewList);
 	setbApplyToCurrent(bApplyToCurrent);
@@ -520,7 +524,7 @@ void AP_Win32Dialog_Lists::_onApply()
 	_win32Dialog.checkButton(AP_RID_DIALOG_LIST_RADIO_START_NEW_LIST, false);
 	_win32Dialog.checkButton(AP_RID_DIALOG_LIST_RADIO_APPLY_TO_CURRENT_LIST, true);
 	_win32Dialog.checkButton(AP_RID_DIALOG_LIST_RADIO_RESUME_PREV_LIST, false);
-	
+
 	_enableControls();
 
 	PopulateDialogData();
@@ -576,7 +580,7 @@ void AP_Win32Dialog_Lists::_fillTypeList()
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_if_fail (pSS);	// TODO: Would an error handler be more appropriate here?
 
 	static const XAP_String_Id rgIDs[] =
 	{
@@ -594,7 +598,7 @@ void AP_Win32Dialog_Lists::_fillTypeList()
 
 void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 {
-	UT_ASSERT(iType >= -1 && iType <=2);
+	UT_return_if_fail (iType >= -1 && iType <=2);
 
 	const UT_sint32 idComboStyle = AP_RID_DIALOG_LIST_COMBO_STYLE;
 
@@ -628,12 +632,9 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 		AP_STRING_ID_DLG_Lists_Lower_Case_List,
 		AP_STRING_ID_DLG_Lists_Upper_Case_List,
 		AP_STRING_ID_DLG_Lists_Lower_Roman_List,
-		AP_STRING_ID_DLG_Lists_Upper_Roman_List
-#ifdef BIDI_ENABLED
-		,AP_STRING_ID_DLG_Lists_Arabic_List,
-
+		AP_STRING_ID_DLG_Lists_Upper_Roman_List,
+		AP_STRING_ID_DLG_Lists_Arabic_List,
 		AP_STRING_ID_DLG_Lists_Hebrew_List
-#endif
 	};
 
 	const XAP_String_Id*	pIDs;
@@ -652,10 +653,10 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 	}
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_if_fail (pSS);	// TODO: Would an error handler be more appropriate here?
 
 	HWND hComboStyle = GetDlgItem(m_hThisDlg, AP_RID_DIALOG_LIST_COMBO_STYLE);
-	UT_ASSERT(hComboStyle);
+	UT_return_if_fail (hComboStyle);
 	int nMaxWidth = 0;
 
 	// Get the HDC of the droplist to be able to get the
@@ -663,7 +664,7 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 	HDC hDCcombo = GetDC(hComboStyle);
 	HDC hDC = CreateCompatibleDC(hDCcombo);
 	ReleaseDC(hComboStyle, hDCcombo);
-	
+
 	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
 	SelectObject(hDC, hFont);
 
@@ -696,7 +697,7 @@ void AP_Win32Dialog_Lists::_typeChanged()
 {
 	const int iType = _getTypeComboCurSel();
 
-	const List_Type type =
+	const FL_ListType type =
 		iType == 1 ? BULLETED_LIST :
 		iType == 2 ? NUMBERED_LIST :
 		NOT_A_LIST;
@@ -715,13 +716,9 @@ void AP_Win32Dialog_Lists::_styleChanged()
 	setNewListType(_getListTypeFromCombos());
 	setbisCustomized(false);
 /*
-#ifdef BIDI_ENABLED
-
 	// have to do this, since for instance the decimal delimter can change
 
 	fillUncustomizedValues();
-
-#endif
 */
 	_previewExposed();
 	_setDisplayedData();
@@ -777,7 +774,7 @@ void AP_Win32Dialog_Lists::_enableCustomControls(bool bEnable)
 void AP_Win32Dialog_Lists::_updateCaption()
 {
 	ConstructWindowName();
-	_win32Dialog.setDialogTitle(getWindowName());
+	_win32Dialog.setDialogTitle((LPCSTR)(AP_Win32App::s_fromUTF8ToAnsi( getWindowName())).c_str());	
 }
 
 void AP_Win32Dialog_Lists::_previewExposed()
@@ -846,7 +843,7 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 	if (controlId == -1 || controlId == AP_RID_DIALOG_LIST_EDIT_LIST_ALIGN)
 	{
 		_win32Dialog.getControlText(AP_RID_DIALOG_LIST_EDIT_LIST_ALIGN, szTmp, 30);
-		setfAlign((float)UT_convertDimensionless(szTmp));	
+		setfAlign((float)UT_convertDimensionless(szTmp));
 	}
 
 	if (controlId == -1 || controlId == AP_RID_DIALOG_LIST_EDIT_INDENT_ALIGN)
@@ -860,11 +857,11 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 		controlId == AP_RID_DIALOG_LIST_EDIT_INDENT_ALIGN)
 	{
 		if( getfAlign() < 0.0 )
-		{	
+		{
 			setfAlign(0.0);
 			_win32Dialog.setControlText(AP_RID_DIALOG_LIST_EDIT_INDENT_ALIGN,
 								UT_convertToDimensionlessString(0.0, ".2"));
-		}                                       
+		}
 
 		if ( (getfIndent() + getfAlign()) < 0.0 )
 		{
@@ -900,10 +897,10 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 }
 
 //
-// Returns the List_Type of the currently selected Type/Style combination.
+// Returns the FL_ListType of the currently selected Type/Style combination.
 // Returns NOT_A_LIST on failure.
 //
-List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
+FL_ListType AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 {
 	const int iType  = _getTypeComboCurSel();
 	int iStyle = _getStyleComboCurSel();
@@ -917,19 +914,19 @@ List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 
 	if (iType == 1 /* bullet list */)
 	{
-		return (List_Type)(iStyle + BULLETED_LIST);
+		return (FL_ListType)(iStyle + BULLETED_LIST);
 	}
 
 	if(iStyle >= BULLETED_LIST)
 		iStyle += (OTHER_NUMBERED_LISTS - BULLETED_LIST + 1);
 
 	// Numbered List, but just to make really REALLY sure, we assert it
-	UT_ASSERT(IS_NUMBERED_LIST_TYPE(iStyle));
+	UT_ASSERT_HARMLESS(IS_NUMBERED_LIST_TYPE(iStyle));
 
-	return (List_Type)iStyle;
+	return (FL_ListType)iStyle;
 }
 
-void AP_Win32Dialog_Lists::_setListType(List_Type type)
+void AP_Win32Dialog_Lists::_setListType(FL_ListType type)
 {
 	int iType;
 	int iStyle;
@@ -948,7 +945,7 @@ void AP_Win32Dialog_Lists::_setListType(List_Type type)
 	}
 	else
 	{
-		UT_ASSERT(IS_BULLETED_LIST_TYPE(type));
+		UT_ASSERT_HARMLESS(IS_BULLETED_LIST_TYPE(type));
 		iType = 1;
 		iStyle = type - BULLETED_LIST;
 	}
@@ -981,11 +978,11 @@ void AP_Win32Dialog_Lists::_selectFont()
 	}
 
 	XAP_DialogFactory* pDialogFactory = pFrame->getDialogFactory();
-	UT_ASSERT(pDialogFactory);
+	UT_return_if_fail (pDialogFactory);
 
 	XAP_Dialog_FontChooser* pDialog
 		= (XAP_Dialog_FontChooser *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_FONT));
-	UT_ASSERT(pDialog);
+	UT_ASSERT_HARMLESS(pDialog);
 
 	if (!pDialog)	// runtime failsafe
 	{
@@ -1043,7 +1040,7 @@ void AP_Win32Dialog_Lists::_selectFont()
 	pDialog->setGraphicsContext(pView->getGraphics());
 	pDialog->runModal(pFrame);
 
-	if (pDialog->getAnswer() != XAP_Dialog_FontChooser::a_OK && 
+	if (pDialog->getAnswer() != XAP_Dialog_FontChooser::a_OK &&
 		pDialog->getAnswer() != XAP_Dialog_FontChooser::a_YES)
 	{
 		return;
@@ -1093,14 +1090,14 @@ void AP_Win32Dialog_Lists::_selectFont()
 
 void AP_Win32Dialog_Lists::autoupdateLists(UT_Worker * pWorker)
 {
-	UT_ASSERT(pWorker);
+	UT_return_if_fail (pWorker);
 	// this is a static callback method and does not have a 'this' pointer.
 	AP_Win32Dialog_Lists* pDialog =
 		reinterpret_cast<AP_Win32Dialog_Lists*>(pWorker->getInstanceData());
 	// Handshaking code. Plus only update if something in the document
 	// changed.
 
-	UT_ASSERT(pDialog);
+	UT_ASSERT_HARMLESS(pDialog);
 
 	if (pDialog)	// runtime failsafe
 	{
