@@ -25,7 +25,7 @@
 
 #include "xap_App.h"
 #include "xap_Win32App.h"
-#include "xap_Win32Frame.h"
+#include "xap_Win32FrameImpl.h"
 
 #include "ap_Strings.h"
 #include "ap_Dialog_Id.h"
@@ -33,13 +33,15 @@
 #include "ap_Win32Dialog_Styles.h"
 
 #include "ap_Win32Resources.rc2"
-
+#include "xap_Win32DialogHelper.h"
 #include "fl_DocLayout.h"
 #include "fl_BlockLayout.h"
 #include "fv_View.h"
 #include "pd_Style.h"
 #include "ut_string_class.h"
 #include "gr_Win32Graphics.h"
+#include "pt_PieceTable.h"
+#include "ap_Win32App.h"
 
 /*****************************************************************/
 
@@ -59,6 +61,7 @@ AP_Win32Dialog_Styles::AP_Win32Dialog_Styles(XAP_DialogFactory * pDlgFactory,
 	m_bisNewStyle(true),
 	m_selectToggle(0)
 {
+	m_pAbiPreviewWidget = NULL;	
 }
 
 AP_Win32Dialog_Styles::~AP_Win32Dialog_Styles(void)
@@ -66,25 +69,127 @@ AP_Win32Dialog_Styles::~AP_Win32Dialog_Styles(void)
 	DELETEP(m_pParaPreviewWidget);
 }
 
+
+/*
+	
+*/
+BOOL CALLBACK AP_Win32Dialog_Styles::s_dlgProc(HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam)
+{
+	// This is a static function.
+
+	AP_Win32Dialog_Styles * pThis;
+	switch (msg)
+	{
+	case WM_INITDIALOG:	
+		pThis = (AP_Win32Dialog_Styles *)lParam;		
+		SetWindowLong(hWnd,DWL_USER,lParam);
+		return pThis->_onInitDialog(hWnd,wParam,lParam);
+
+	case WM_COMMAND:
+		
+		pThis = (AP_Win32Dialog_Styles *)GetWindowLong(hWnd,DWL_USER);
+		if(pThis)
+			return pThis->_onCommand(hWnd,wParam,lParam);
+		else		
+			return 0;
+			
+	case WM_DRAWITEM:		
+		AP_Win32Dialog_Styles * pThis;
+		pThis = (AP_Win32Dialog_Styles *)lParam;				
+		
+		if (wParam==AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS)
+			pThis->_onDrawButton((LPDRAWITEMSTRUCT)lParam, hWnd);
+		
+		return 0;
+	
+	default:
+		return 0;
+	}
+}
+
+
+/*
+	Draws the Format button with an arrow
+*/
+void AP_Win32Dialog_Styles::_onDrawButton(LPDRAWITEMSTRUCT lpDrawItemStruct, HWND hWnd)
+{
+    UINT			uiState    = lpDrawItemStruct->itemState;
+    HPEN			hPen;
+    HPEN			pOldPen;
+	HDC				hdc = lpDrawItemStruct->hDC;
+	int				nWidth;
+	int				nHeight;
+	int		 		x, xEnd, xStart;
+	int 			y;
+	POINT 			p;	
+	const char* 	pText;
+	HWND 			hParent;
+	LONG			lData;
+	
+	// Get parent object and get the App object from there
+	hParent = GetParent(hWnd);	
+	lData = GetWindowLong(hWnd,DWL_USER);			
+	AP_Win32Dialog_Styles * pParent = (AP_Win32Dialog_Styles *)lData;
+	const XAP_StringSet * pSS = pParent->m_pApp->getStringSet();		
+	
+	pText=	pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyFormat);		
+	
+	nWidth = lpDrawItemStruct->rcItem.right - lpDrawItemStruct->rcItem.left;
+	nHeight = lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top;
+		      
+    // set the pen color
+    if (uiState&ODS_DISABLED)
+        hPen = CreatePen(PS_SOLID, 0, GetSysColor(COLOR_GRAYTEXT));
+    else
+        hPen = CreatePen(PS_SOLID, 0, GetSysColor(COLOR_BTNTEXT));
+    
+    pOldPen =  (HPEN) SelectObject(hdc, hPen);
+
+    // draw the border of the button
+    if(uiState&ODS_SELECTED)
+        DrawFrameControl(hdc, &lpDrawItemStruct->rcItem, DFC_BUTTON, DFCS_BUTTONPUSH|DFCS_PUSHED);
+    else
+        DrawFrameControl(hdc, &lpDrawItemStruct->rcItem, DFC_BUTTON, DFCS_BUTTONPUSH);	
+	
+	// Draw arrow
+	y = nHeight/2;
+	xStart = (nWidth/6)*5;
+	for (int i=0; i<4; i++)
+	{
+  	   x = xStart + i;
+	   xEnd = xStart + 7 - i;
+
+	   ::MoveToEx(hdc, x, y, &p);	   
+       ::LineTo(hdc, xEnd, y);
+	   y++;
+	 }
+
+	ExtTextOut(hdc, (nWidth/6)*1, ((nHeight/4)), 0, NULL, pText,  strlen(pText), NULL);
+		
+    // Clean Up
+    SelectObject(hdc, pOldPen);       
+    DeleteObject(hPen);
+}
+
 void AP_Win32Dialog_Styles::runModal(XAP_Frame * pFrame)
 {
-	UT_ASSERT(pFrame);
+	UT_return_if_fail (pFrame);
 //
 // Get View and Document pointers. Place them in member variables
 //
 	setFrame(pFrame);
 
 	setView((FV_View *) pFrame->getCurrentView());
-	UT_ASSERT(getView());
+	UT_return_if_fail (getView());
 
 	setDoc(getView()->getLayout()->getDocument());
-	UT_ASSERT(getDoc());
+	UT_return_if_fail (getDoc());
 
 
 	// raise the dialog
-
 	_win32Dialog.runModal(pFrame, AP_DIALOG_ID_STYLES, AP_RID_DIALOG_STYLES_TOP, this);
-
+	
+	
 	if (m_answer == AP_Dialog_Styles::a_OK)
 	{
 		const char* szStyle = getCurrentStyle();
@@ -101,17 +206,16 @@ void AP_Win32Dialog_Styles::runModal(XAP_Frame * pFrame)
 BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
 	XAP_Win32App * app = static_cast<XAP_Win32App *> (m_pApp);
-	UT_ASSERT(app);
+	UT_return_val_if_fail (app,0);
 	
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	char szTemp[20];
-	GetWindowText(hWnd, szTemp, 20 );
-
+	GetWindowText(hWnd, szTemp, 20 );	
+			
+	// Regular dialog box
 	if( strncmp(szTemp, "Styles", 20) == 0 )
-	{
-		// m_hThisDlg = hWnd;
-	
+	{	
 		SetWindowText(hWnd, pSS->getValue(AP_STRING_ID_DLG_Styles_StylesTitle));
 
 		// localize controls
@@ -172,12 +276,15 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 
 		_populateWindowData();
 	}
-	else  // This is either the new or Modify sub dialog of styles
+	// This is either the new or Modify sub dialog of styles
+	else  
 	{
 		// Localize the controls Labels etc...
 		SetWindowText(hWnd, pSS->getValue( (m_bisNewStyle) ? 
                                            AP_STRING_ID_DLG_Styles_NewTitle :
                                            AP_STRING_ID_DLG_Styles_ModifyTitle ));
+
+		_win32DialogNewModify.setHandle(hWnd);
 		
 		#define _DS(c,s)  _win32DialogNewModify.setControlText(AP_RID_DIALOG_##c,pSS->getValue(AP_STRING_ID_##s))
 		#define _DSX(c,s)  _win32DialogNewModify.setControlText(AP_RID_DIALOG_##c,pSS->getValue(XAP_STRING_ID_##s))
@@ -188,14 +295,14 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 		_DS(STYLES_NEWMODIFY_LBL_REMOVE,		DLG_Styles_RemoveLab);
 		_DS(STYLES_NEWMODIFY_GBX_PREVIEW,		DLG_Styles_ModifyPreview);
 		_DS(STYLES_NEWMODIFY_GBX_DESC,			DLG_Styles_ModifyDescription);
-		_DS(STYLES_NEWMODIFY_BTN_REMOVE,		DLG_Styles_RemoveButton);
-		_DS(STYLES_NEWMODIFY_BTN_TOGGLEITEMS,	DLG_Styles_ModifyParagraph);
+		_DS(STYLES_NEWMODIFY_BTN_REMOVE,		DLG_Styles_RemoveButton);		
 		_DS(STYLES_NEWMODIFY_BTN_SHORTCUT,		DLG_Styles_ModifyShortCut);
 		_DSX(STYLES_NEWMODIFY_BTN_OK,			DLG_OK);
 		_DSX(STYLES_NEWMODIFY_BTN_CANCEL,		DLG_Cancel);
 		#undef _DSX
 		#undef _DS
-
+		
+		
 		// Changes basic controls based upon either New or Modify Dialog
 		_win32DialogNewModify.showControl( AP_RID_DIALOG_STYLES_NEWMODIFY_EBX_TYPE , 
                                            (m_bisNewStyle) ? SW_HIDE : SW_SHOW );
@@ -205,36 +312,71 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 
 		size_t nStyles = getDoc()->getStyleCount();
 		const char * name = NULL;
+		const char * pLocalised = NULL;
 		const PD_Style * pcStyle = NULL;
+		int nIndex;
+		UT_UTF8String utf8;
+		UT_String str;	
+		
 		for (UT_uint32 i = 0; i < nStyles; i++)
 		{
-    		getDoc()->enumStyles(i, &name, &pcStyle);
-			_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, name );
-			_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, name );
+    		getDoc()->enumStyles(i, &name, &pcStyle);	
+    		
+   			pt_PieceTable::s_getLocalisedStyleName(name, utf8);			
+			pLocalised = utf8.utf8_str();
+			str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+			pLocalised = str.c_str();	
+			
+			nIndex = _win32DialogNewModify.addItemToCombo(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, pLocalised);				
+			_win32DialogNewModify.setComboDataItem(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, 
+				nIndex, i);				
+			
+			nIndex = _win32DialogNewModify.addItemToCombo(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, pLocalised);				
+			_win32DialogNewModify.setComboDataItem(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, 
+				nIndex, i);				
 		}
+		
+		// Strings (not styles names)
+		const char*	pDefCurrent = pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent);
+		const char*	pDefNone = pSS->getValue(AP_STRING_ID_DLG_Styles_DefNone);
+		
+		str = AP_Win32App::s_fromUTF8ToAnsi(pDefCurrent);
+		pDefCurrent = str.c_str();	
+		
 		_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, 
-                                              pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent) );
+                                              pDefCurrent );
+
+
+		str = AP_Win32App::s_fromUTF8ToAnsi(pDefNone);
+		pDefNone = str.c_str();							
+
 		_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, 
-                                              pSS->getValue(AP_STRING_ID_DLG_Styles_DefNone) );
+                                              pDefNone);
+		
+		
 		if( m_bisNewStyle )
 		{	
-			// Add last Member item which will be defined as the default value
-			//_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, 
-            //                                      pSS->getValue(AP_STRING_ID_DLG_Styles_DefNone) );
-			//_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, 
-            //                                      pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent) );
+			
+			const char* p = pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyParagraph);
+			
 			_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_TYPE,
-                                                  pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyParagraph) );
+                                                  p );
+                                                  
+			p = pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyCharacter);
+                                                  
 			_win32DialogNewModify.addItemToCombo( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_TYPE,
-                                                  pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyCharacter) );
-			// Set the Default Item
+                                                  p);			                         
+                                                  
+			// Set the Default syltes: none, default current
 			UT_sint32 result;
 			result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, CB_FINDSTRING, -1,
 										(LPARAM) pSS->getValue(AP_STRING_ID_DLG_Styles_DefNone));
 			_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, result );
+			
 			result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, CB_FINDSTRING, -1,
 										(LPARAM) pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent));
 			_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, result );
+			
 			result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_TYPE, CB_FINDSTRING, -1,
 										(LPARAM) pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyParagraph));
 			_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_TYPE, result );
@@ -249,17 +391,27 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 			const char * szCurrentStyle = NULL;
 			const char * szBasedOn = NULL;
 			const char * szFollowedBy = NULL;
+			const char * pLocalised = NULL;
 			PD_Style * pStyle = NULL;
 			PD_Style * pBasedOnStyle = NULL;
 			PD_Style * pFollowedByStyle = NULL;
+			UT_UTF8String utf8;
+			UT_String str;	
 			
 			szCurrentStyle = m_selectedStyle.c_str();
+			
+			pt_PieceTable::s_getLocalisedStyleName(szCurrentStyle, utf8);						
+			pLocalised = utf8.utf8_str();
+			str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+			pLocalised = str.c_str();
+	
 		
 			_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_EBX_NAME,
-                                                  (char *) szCurrentStyle );
+                                                  pLocalised);
                                                   
 			if(szCurrentStyle)
 				getDoc()->getStyle(szCurrentStyle,&pStyle);
+				
 			if(!pStyle)
 			{
 				XAP_Frame * pFrame = getFrame();
@@ -278,6 +430,7 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 			size_t nStyles = getDoc()->getStyleCount();
 			const char * name = NULL;
 			const PD_Style * pcStyle = NULL;
+	
 			for (UT_uint32 i = 0; i < nStyles; i++)
 			{
 			    getDoc()->enumStyles(i, &name, &pcStyle);
@@ -294,27 +447,48 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 		
 			if(pBasedOnStyle != NULL)
 			{
+
+				pt_PieceTable::s_getLocalisedStyleName(szBasedOn, utf8);
+				pLocalised = utf8.utf8_str();
+				str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+				pLocalised = str.c_str();
+	
+				
 				UT_uint32 result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, CB_FINDSTRING, -1,
-										(LPARAM) szBasedOn);
+										(LPARAM)pLocalised);
+										
 				_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, result );
 			}
 			else
 			{
+				// Not a style name
 				UT_uint32 result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, CB_FINDSTRING, -1,
 										(LPARAM) pSS->getValue(AP_STRING_ID_DLG_Styles_DefNone));
+										
 				_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, result );
 			}
 
 			if(pFollowedByStyle != NULL)
 			{
+				pt_PieceTable::s_getLocalisedStyleName(szFollowedBy, utf8);		
+				pLocalised = utf8.utf8_str();
+				str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+				pLocalised = str.c_str();
+				
 				UT_uint32 result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, CB_FINDSTRING, -1,
-										(LPARAM) szFollowedBy);
+										(LPARAM)pLocalised);
 				_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, result );
 			}
 			else
 			{
+				pt_PieceTable::s_getLocalisedStyleName(pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent), utf8);		
+				pLocalised = utf8.utf8_str();
+				str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+				pLocalised = str.c_str();
+
+				
 				UT_uint32 result = SendDlgItemMessage(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, CB_FINDSTRING, -1,
-										(LPARAM) pSS->getValue(AP_STRING_ID_DLG_Styles_DefCurrent));
+										(LPARAM) pLocalised);
 				_win32DialogNewModify.selectComboItem( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, result );
 			}
 			
@@ -352,6 +526,8 @@ BOOL AP_Win32Dialog_Styles::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPara
 		_populatePreviews(true);
 
 	}
+	
+	XAP_Win32DialogHelper::s_centerDialog(hWnd);	
 	return 1;							// 1 == we did not call SetFocus()
 }
 
@@ -388,6 +564,8 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 			_win32DialogNewModify.getControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_EBX_NAME,
                                                   m_newStyleName,
 	                                              MAX_EBX_LENGTH );
+			
+
 			if( !m_newStyleName || !strlen(m_newStyleName) )
 			{
 			    getFrame()->showMessageBox( pSS->getValue (AP_STRING_ID_DLG_Styles_ErrBlankName),
@@ -396,7 +574,8 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 
 			    return 1;
     		}
-			// Verfiy 
+
+			strcpy (m_newStyleName, (AP_Win32App::s_fromAnsiToUTF8(m_newStyleName)).utf8_str());
 
 		}
 		m_answer = a_OK;
@@ -429,24 +608,27 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 	case AP_RID_DIALOG_STYLES_TOP_LIST_STYLES:
 		if (wNotifyCode == LBN_SELCHANGE)
 		{
-			int row = _win32Dialog.getListSelectedIndex(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES);
-
-			if (row == -1)
+			
+			UT_uint32	nData = -1;
+			const char* name;
+			const PD_Style * pcStyle = NULL;
+			
+			int row = _win32Dialog.getListSelectedIndex(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES);					
+			
+			nData = _win32Dialog.getListDataItem( AP_RID_DIALOG_STYLES_TOP_LIST_STYLES, row);
+			
+			if (row!=LB_ERR)
 			{
-				m_selectedStyle = "";
-				return 1;
+				getDoc()->enumStyles(nData, &name, &pcStyle);				
+				m_selectedStyle = name; 
+				
+				m_nSelectedStyleIdx = nData;
+							
+				// refresh the previews
+				_populatePreviews(false);	
 			}
-
-
-			char *p_buffer = new char [1024];
-
-			_win32Dialog.getListText(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES, row, p_buffer);
-
-			m_selectedStyle = p_buffer;
-			delete [] p_buffer;
-
-			// refresh the previews
-			_populatePreviews(false);
+			
+			break;			
 		}
 		return 1;
 
@@ -506,9 +688,15 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 				getDoc()->getStyle(m_selectedStyle.c_str(), &pStyle);
 
 				m_bisNewStyle = false;
-				XAP_Frame* pFrame = getFrame();
+				XAP_Frame* pFrame = getFrame();			
+				
+				XAP_Win32App * pWin32App = static_cast<XAP_Win32App *>(getApp());
 			
-				_win32DialogNewModify.runModal(pFrame, AP_DIALOG_ID_STYLES, AP_RID_DIALOG_STYLES_NEWMODIFY, this);
+				LPCTSTR lpTemplate = MAKEINTRESOURCE(AP_RID_DIALOG_STYLES_NEWMODIFY);				
+											
+				int result = DialogBoxParam(pWin32App->getInstance(), lpTemplate,
+									static_cast<XAP_Win32FrameImpl*>(pFrame->getFrameImpl())->getTopLevelWindow(),
+									(DLGPROC)s_dlgProc, (LPARAM)this);
 				
 				if(m_answer == AP_Dialog_Styles::a_OK)
 				{
@@ -525,10 +713,10 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 
 	case AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_REMOVE:
 		{
-			char szTemp[40];
+			char szTemp[128];
 			_win32DialogNewModify.getControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_REMOVE,
                                                   szTemp,
-	                                              40 );			
+	                                              sizeof(szTemp) );			
 			removeVecProp(szTemp);
 			rebuildDeleteProps();
 			updateCurrentStyle();
@@ -536,27 +724,61 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		return 1;
 
 	case AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS:
-		switch( m_selectToggle )
+	{
+	    RECT 	rect;
+	    HMENU 	hMenu;
+	    int		x,y;	    	    
+	    HWND	hWndButton;
+	    
+	    hWndButton = GetDlgItem(hWnd, AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS);
+	    XAP_Win32App * app = static_cast<XAP_Win32App *> (m_pApp);		
+		const XAP_StringSet * pSS = m_pApp->getStringSet();
+	    
+		// Get button position
+	    GetWindowRect(hWndButton, &rect);
+	    x = rect.left;
+	    y = rect.bottom;	               		
+
+	    // Menu creation
+	    hMenu =  CreatePopupMenu();
+	    AppendMenu(hMenu, MF_ENABLED|MF_STRING, 1, (LPCTSTR)pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyParagraph));
+	    AppendMenu(hMenu, MF_ENABLED|MF_STRING, 2, (LPCTSTR)pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyFont));
+	    AppendMenu(hMenu, MF_ENABLED|MF_STRING, 3, (LPCTSTR)pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyTabs));
+	    AppendMenu(hMenu, MF_ENABLED|MF_STRING, 4, (LPCTSTR)pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyNumbering));
+	    AppendMenu(hMenu, MF_ENABLED|MF_STRING, 5, (LPCTSTR)pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyLanguage));
+	    
+	    // show and track the menu
+    	m_selectToggle = TrackPopupMenu(hMenu, TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_NONOTIFY|TPM_RETURNCMD,
+    						x,y,0, hWndButton,  NULL);		    							    	        						 							    
+	    
+	    switch(m_selectToggle)
 		{
-		case 0:
-			ModifyParagraph();
+		case 0:	// user has cancelled
 			break;
 		case 1:
-			ModifyFont();
+			ModifyParagraph();
 			break;
 		case 2:
-			ModifyTabs();
+			ModifyFont();
 			break;
 		case 3:
-			ModifyLists();
+			ModifyTabs();
 			break;
 		case 4:
+			ModifyLists();
+			break;
+		case 5:
 			ModifyLang();
 			break;
+		default:
+			break;			
 		}
+		
 		rebuildDeleteProps();
 		updateCurrentStyle();
+	    DestroyMenu(hMenu);
 		return 1;
+	}
 
 
 	case AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON:
@@ -585,22 +807,10 @@ BOOL AP_Win32Dialog_Styles::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		UT_DEBUGMSG(("WM_Command for id %ld\n",wId));
 		return 0;						// return zero to let windows take care of it.
 	}
+	
+	return 0;						// return zero to let windows take care of it.
 }
 
-BOOL AP_Win32Dialog_Styles::_onDeltaPos(NM_UPDOWN * pnmud)
-{
-	switch( pnmud->hdr.idFrom )
-	{
-	case AP_RID_DIALOG_STYLES_NEWMODIFY_SPN_TOGGLEITEMS: 
-		m_selectToggle += pnmud->iDelta;
-		if( m_selectToggle > MAX_NEWMODIFY_TOGGLE ) m_selectToggle = 0;
-		if( m_selectToggle < 0 ) m_selectToggle = MAX_NEWMODIFY_TOGGLE;
-		_updateToggleButtonText();
-		return 1;
-	default:
-		return 0;
-	}
-}
 
 void AP_Win32Dialog_Styles::_populateWindowData(void)
 {
@@ -612,12 +822,17 @@ void AP_Win32Dialog_Styles::_populateCList(void)
 {
 	const PD_Style * pStyle;
 	const char * name = NULL;
+	const char*	pLocalised = NULL;
+	int nIndex;
+	UT_UTF8String utf8;
+	UT_String str;						 
 
 	size_t nStyles = getDoc()->getStyleCount();
 	xxx_UT_DEBUGMSG(("DOM: we have %d styles\n", nStyles));
 
 	_win32Dialog.resetContent(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES);
 	
+	
 	for (UT_uint32 i = 0; i < nStyles; i++)
 	{
 	    const char * data[1];
@@ -629,13 +844,21 @@ void AP_Win32Dialog_Styles::_populateCList(void)
 			continue;
 
 	    // all of this is safe to do... append should take a const char **
-	    data[0] = name;
+	    data[0] = name;	    
 
 	    if ((m_whichType == ALL_STYLES) || 
 			(m_whichType == USED_STYLES && pStyle->isUsed()) ||
 			(m_whichType == USER_STYLES && pStyle->isUserDefined()))
 		{
-			_win32Dialog.addItemToList(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES, *data);
+
+			pt_PieceTable::s_getLocalisedStyleName(*data, utf8);
+			pLocalised = utf8.utf8_str();
+			str = AP_Win32App::s_fromUTF8ToAnsi(pLocalised);
+			pLocalised = str.c_str();
+			
+			nIndex = _win32Dialog.addItemToList(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES, pLocalised);						
+			_win32Dialog.setListDataItem(AP_RID_DIALOG_STYLES_TOP_LIST_STYLES, nIndex, i);							
+			
 		}
 	}
 
@@ -660,38 +883,6 @@ void AP_Win32Dialog_Styles::setModifyDescription (const char * desc)
 	p_This->_win32DialogNewModify.setControlText(AP_RID_DIALOG_STYLES_NEWMODIFY_CTL_DESC, desc);
 }
 
-void AP_Win32Dialog_Styles::_updateToggleButtonText()
-{
-	XAP_Win32App * app = static_cast<XAP_Win32App *> (m_pApp);
-	UT_ASSERT(app);
-	const XAP_StringSet * pSS = m_pApp->getStringSet();
-
-	AP_Win32Dialog_Styles *p_This = (AP_Win32Dialog_Styles *)this; // Cast away const
-
-	switch(m_selectToggle)
-	{
-	case 0:
-		p_This->_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS,
-                                                      pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyParagraph) );
-		break;
-	case 1:
-		p_This->_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS,
-                                                      pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyFont) );
-		break;
-	case 2:
-		p_This->_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS,
-                                                      pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyTabs) );
-		break;
-	case 3:
-		p_This->_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS,
-                                                      pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyNumbering) );
-		break;
-	case 4:
-		p_This->_win32DialogNewModify.setControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_BTN_TOGGLEITEMS,
-                                                      pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyLanguage) );
-		break;
-	}
-}	
 
 void AP_Win32Dialog_Styles::rebuildDeleteProps()
 {
@@ -709,33 +900,61 @@ void AP_Win32Dialog_Styles::rebuildDeleteProps()
 
 void AP_Win32Dialog_Styles::eventBasedOn()
 {
-	char szTemp[40];
-	_win32DialogNewModify.getControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON,
-                                          szTemp,
-	                                      40 );			
-	addOrReplaceVecAttribs("basedon",szTemp);
-	fillVecWithProps(szTemp,false);
+	char szTemp[128];
+	int	nSel;
+	int	nData;
+	const PD_Style * pStyle = NULL;
+	const char* pText = szTemp;
+		
+	nSel = _win32DialogNewModify.getComboSelectedIndex(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON);		
+				
+	if (nSel==CB_ERR) return;				
+	
+	nData= _win32DialogNewModify.getComboDataItem(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_BASEDON, nSel);
+	
+	size_t nStyles = getDoc()->getStyleCount();							
+	
+	getDoc()->enumStyles((UT_uint32)nData,&pText, &pStyle);				
+	
+	strcpy (szTemp, (AP_Win32App::s_fromAnsiToUTF8(pText)).utf8_str());
+	addOrReplaceVecAttribs("basedon",pText);
+	fillVecWithProps(pText,false);
 	updateCurrentStyle();
 }
 
 void AP_Win32Dialog_Styles::eventFollowedBy()
 {
-	char szTemp[40];
-	_win32DialogNewModify.getControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA,
-                                          szTemp,
-	                                      40 );			
-	addOrReplaceVecAttribs("followedby",szTemp);
+	char szTemp[128]; 
+	int	nSel;
+	int	nData;
+	const PD_Style * pStyle = NULL;
+	const char* pText = szTemp;
+		
+	nSel = _win32DialogNewModify.getComboSelectedIndex(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA);		
+					
+	if (nSel==CB_ERR) return;				
+	
+	nData= _win32DialogNewModify.getComboDataItem(AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_FOLLOWPARA, nSel);
+					
+	size_t nStyles = getDoc()->getStyleCount();							
+	
+	getDoc()->enumStyles((UT_uint32)nData,&pText, &pStyle);			
+	
+	strcpy (szTemp, (AP_Win32App::s_fromAnsiToUTF8(pText)).utf8_str());
+	addOrReplaceVecAttribs("followedby",pText);
 }
 
 void AP_Win32Dialog_Styles::eventStyleType()
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();	
 	const XML_Char * pszSt = "P";
-	char szTemp[40];
+	char szTemp[128];
 	_win32DialogNewModify.getControlText( AP_RID_DIALOG_STYLES_NEWMODIFY_CBX_TYPE,
                                           szTemp,
-                                          40 );			
+                                          sizeof(szTemp));			
 	if(strstr(szTemp, pSS->getValue(AP_STRING_ID_DLG_Styles_ModifyCharacter)) != 0)
 		pszSt = "C";
+	
 	addOrReplaceVecAttribs("type",pszSt);
 }
+
