@@ -27,7 +27,8 @@
 #include "ap_Strings.h"
 #include "ut_string.h"
 #include "ut_string_class.h"
-
+#include "pp_AttrProp.h"
+#include "xap_App.h"
 #include "xap_Dialog_Id.h"
 #include "xap_DialogFactory.h"
 #include "xap_Dlg_MessageBox.h"
@@ -39,7 +40,9 @@
 #include "fl_BlockLayout.h"
 #include "ap_Preview_Paragraph.h"
 #include "xad_Document.h"
-#include <fribidi.h>
+#include "gr_Painter.h"
+#include "pf_Frag_Strux_Block.h"
+
 
 AP_Dialog_Lists::AP_Dialog_Lists(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
 :	XAP_Dialog_Modeless(pDlgFactory, id, "interface/dialoglists"),
@@ -95,6 +98,8 @@ AP_Dialog_Lists::~AP_Dialog_Lists(void)
 	for(UT_uint32 i=0; i<4; i++)
 	{
 		DELETEP(m_pFakeLayout[i]);
+		const pf_Frag_Strux_Block * pFSB = static_cast<const pf_Frag_Strux_Block *>(m_pFakeSdh[i]);
+		DELETEP(pFSB);
 	}
 	// What do we do about the fakeAutoNum in the Document pDoc?
 	// Maybe we need another constrcutor
@@ -122,7 +127,7 @@ void AP_Dialog_Lists::_createPreviewFromGC(GR_Graphics* gc,
 										   UT_uint32 width,
 										   UT_uint32 height)
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 
 	m_iWidth = width;
 	m_iHeight = height;
@@ -132,7 +137,7 @@ void AP_Dialog_Lists::_createPreviewFromGC(GR_Graphics* gc,
 	DELETEP(m_pListsPreview);
 
 	m_pListsPreview = new AP_Lists_preview(gc, this);
-	UT_ASSERT(m_pListsPreview);
+	UT_return_if_fail (m_pListsPreview);
 
 	m_pListsPreview->setWindowSize(width, height);
 	//
@@ -159,16 +164,16 @@ void AP_Dialog_Lists::event_PreviewAreaExposed(void)
 	}
 	else
 	{
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 	}
 }
 
 void AP_Dialog_Lists::StartList(void)
 {
-	UT_ASSERT(!IS_NONE_LIST_TYPE(m_DocListType));
+	UT_ASSERT_HARMLESS(!IS_NONE_LIST_TYPE(m_DocListType));
 	getBlock()->listUpdate();
 	const XML_Char* pStyle = getBlock()->getListStyleString(m_DocListType);
-	UT_ASSERT(pStyle);
+	UT_return_if_fail (pStyle);
 	getView()->cmdStartList(pStyle);
 }
 
@@ -218,6 +223,32 @@ void AP_Dialog_Lists::setTick(UT_uint32 iTick)
 void AP_Dialog_Lists::Apply(void)
 {
 	XML_Char szStart[20];
+	if(!isModal() && !isPageLists())
+	{
+//
+// OK fold up the text according the level specified.
+//
+		fl_AutoNum * pAuto = getBlock()->getAutoNum();
+		if(!pAuto)
+		{
+			return;
+		}
+		const XML_Char * props[5] = {"text-folded",NULL,"text-folded-id",NULL,NULL};
+		UT_UTF8String sStr = UT_UTF8String_sprintf("%d",getCurrentFold());
+		props[1] = sStr.utf8_str();
+		PT_DocPosition posLow = 0;
+		PT_DocPosition posHigh = 0;
+		UT_uint32 ID = pAuto->getID();
+		UT_UTF8String sID = UT_UTF8String_sprintf("%d",ID);
+		props[3] = sID.utf8_str();
+		PL_StruxDocHandle sdhLow = pAuto->getFirstItem();
+		PL_StruxDocHandle sdhHigh = pAuto->getLastItem();
+		posLow = getView()->getDocument()->getStruxPosition(sdhLow)+1;
+		posHigh = getView()->getDocument()->getStruxPosition(sdhHigh)+1;
+		getView()->setCollapsedRange(posLow,posHigh,props);
+		return;
+	}
+
 /*!
  *
  * OK this is failsafe code incase the user has changed the font but wants a
@@ -266,6 +297,7 @@ void AP_Dialog_Lists::Apply(void)
 		m_OutProps.addItem((void *) "text-indent");
 		m_Output[3] = UT_convertInchesToDimensionString(DIM_IN, m_fIndent, 0);
 		m_OutProps.addItem((void *) m_Output[3].c_str());
+		m_Answer = a_OK;
 		return;
 	}
 /*!
@@ -275,18 +307,20 @@ void AP_Dialog_Lists::Apply(void)
  * This piece of code changes the list style at the current point to the
  * Style requested by the user.
  */
-	UT_Vector vBlock;
+	UT_GenericVector<fl_BlockLayout*> vBlock;
 	UT_uint32 i = 0;
 	getView()->getBlocksInSelection(&vBlock);
 	UT_uint32 count = vBlock.getItemCount();
 	getView()->cmdUnselectSelection();
 	if(m_bApplyToCurrent == true && m_isListAtPoint == true &&  m_NewListType != NOT_A_LIST)
 	{
+		getView()->getDocument()->beginUserAtomicGlob();
 		getView()->changeListStyle(getAutoNum(),m_NewListType,m_iStartValue,(XML_Char *) m_pszDelim,(XML_Char *) m_pszDecimal, m_pszFont,m_fAlign,m_fIndent);
 		if(getAutoNum() != NULL)
 		{
 			getAutoNum()->update(0);
 		}
+		getView()->getDocument()->endUserAtomicGlob();
 		clearDirty();
 		return;
 	}
@@ -295,6 +329,7 @@ void AP_Dialog_Lists::Apply(void)
  */
 	if ( m_isListAtPoint == true &&  m_NewListType == NOT_A_LIST)
 	{
+		getView()->getDocument()->beginUserAtomicGlob();
 		for(i=0;i < count; i++)
 		{
 			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
@@ -303,6 +338,7 @@ void AP_Dialog_Lists::Apply(void)
 				getView()->getDocument()->StopList(pBlock->getStruxDocHandle());
 			}
 		}
+		getView()->getDocument()->endUserAtomicGlob();
 		clearDirty();
 		return;
 	}
@@ -322,6 +358,7 @@ void AP_Dialog_Lists::Apply(void)
  */
 	if(m_bStartNewList == true)
 	{
+		getView()->getDocument()->beginUserAtomicGlob();
 		for(i=0;i < count; i++)
 		{
 			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
@@ -386,6 +423,7 @@ void AP_Dialog_Lists::Apply(void)
 					{
 						pBlock->resumeList(rBlock);
 						pBlock->getDocument()->enableListUpdates();
+						pBlock->getDocument()->updateDirtyLists();
 					}
 				}
 			}
@@ -405,6 +443,9 @@ void AP_Dialog_Lists::Apply(void)
 			}
 		}
 		clearDirty();
+		getView()->updateScreen(true);
+		getView()->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR);
+		getView()->getDocument()->endUserAtomicGlob();
 		return;
 	}
 /*!
@@ -412,6 +453,7 @@ void AP_Dialog_Lists::Apply(void)
  */
 	if(m_bResumeList == true &&  m_isListAtPoint != true )
 	{
+		getView()->getDocument()->beginUserAtomicGlob();
 		for(i=0;i < count; i++)
 		{
 			fl_BlockLayout * pBlock = (fl_BlockLayout *) vBlock.getNthItem(i);
@@ -423,7 +465,10 @@ void AP_Dialog_Lists::Apply(void)
 				pBlock->getDocument()->updateDirtyLists();
 			}
 		}
+		getView()->getDocument()->endUserAtomicGlob();
 	}
+	getView()->updateScreen(true);
+	getView()->notifyListeners(AV_CHG_MOTION | AV_CHG_HDRFTR);
 	clearDirty();
 }
 
@@ -604,7 +649,8 @@ void  AP_Dialog_Lists::generateFakeLabels(void)
 	for(i=0; i<4; i++)
 	{
 		DELETEP(m_pFakeLayout[i]);
-		m_pFakeSdh[i] = (PL_StruxDocHandle) fakeApp++;
+		m_pFakeSdh[i] = static_cast<PL_StruxDocHandle>(new pf_Frag_Strux_Block(NULL,0));
+		static_cast<const pf_Frag *>(m_pFakeSdh[i])->setPos(i);
 		m_pFakeLayout[i] = new fl_Layout((PTStruxType) 0 , (PL_StruxDocHandle) m_pFakeSdh[i] );
 	}
 	//
@@ -613,7 +659,8 @@ void  AP_Dialog_Lists::generateFakeLabels(void)
 	DELETEP(m_pFakeAuto);
 	UNREFP(m_pFakeDoc);
 	m_pFakeDoc = new PD_Document(fakeApp);
-	m_pFakeAuto = new fl_AutoNum(m_iID, 0, m_NewListType, m_newStartValue, m_pszDelim, m_pszDecimal, (PD_Document *) m_pFakeDoc);
+	m_pFakeAuto = new fl_AutoNum(m_iID, 0, m_NewListType, m_newStartValue, m_pszDelim, m_pszDecimal,
+								 (PD_Document *) m_pFakeDoc,NULL);
 	m_pFakeAuto->insertFirstItem(m_pFakeSdh[0], NULL,1,false);
 	m_pFakeLayout[0]->setAutoNum(m_pFakeAuto);
 	for(i=1; i<4; i++)
@@ -629,7 +676,7 @@ void  AP_Dialog_Lists::generateFakeLabels(void)
  */
 UT_UCSChar * AP_Dialog_Lists::getListLabel(UT_sint32 itemNo)
 {
-	UT_ASSERT(itemNo < 4);
+	UT_ASSERT_HARMLESS(itemNo < 4);
 	static UT_UCSChar lab[80];
 	const UT_UCSChar * tmp  = m_pFakeAuto->getLabel(m_pFakeSdh[itemNo]);
 	if(tmp == NULL)
@@ -648,7 +695,7 @@ UT_UCSChar * AP_Dialog_Lists::getListLabel(UT_sint32 itemNo)
  * Fill our variables from this vector.
  * This is used by the Modal dialog and is filled from the styles dialog.
  */
-void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
+void AP_Dialog_Lists::fillDialogFromVector( UT_GenericVector<const XML_Char*> * vp)
 {
 	UT_sint32 i;
 	if(vp->getItemCount() > 0)
@@ -656,7 +703,7 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
 		i = findVecItem(vp,"start-value");
 		if(i >= 0)
 		{
-			m_iStartValue = atoi( (const XML_Char *) vp->getNthItem(i+1));
+			m_iStartValue = atoi(vp->getNthItem(i+1));
 		}
 		else
 		{
@@ -666,7 +713,7 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
 		i = findVecItem(vp,"margin-left");
 		if(i>=0)
 		{
-			m_fAlign = (float)UT_convertToInches((const XML_Char *) vp->getNthItem(i+1));
+			m_fAlign = (float)UT_convertToInches(vp->getNthItem(i+1));
 		}
 		else
 		{
@@ -676,7 +723,7 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
 		i = findVecItem(vp,"text-indent");
 		if(i >= 0)
 		{
-			m_fIndent = (float)UT_convertToInches((const XML_Char *) vp->getNthItem(i+1));
+			m_fIndent = (float)UT_convertToInches(vp->getNthItem(i+1));
 		}
 		else
 		{
@@ -686,35 +733,35 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
 		i = findVecItem(vp,"list-delim");
 		if( i>= 0)
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) vp->getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, vp->getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
+			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, "%L");
 		}
 		i = findVecItem(vp,"list-decimal");
 		if( i>= 0)
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) vp->getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, vp->getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
+			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, ".");
 		}
 
 		i = findVecItem(vp,"field-font");
 		if( i>= 0)
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) vp->getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, vp->getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
+			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, "NULL");
 		}
 		i = findVecItem(vp,"list-style");
 		if( i>= 0)
 		{
-			m_DocListType = getBlock()->getListTypeFromStyle( (const XML_Char *) vp->getNthItem(i+1));
+			m_DocListType = getBlock()->getListTypeFromStyle(vp->getNthItem(i+1));
 			m_NewListType = m_DocListType;
 		}
 		else
@@ -731,7 +778,7 @@ void AP_Dialog_Lists::fillDialogFromVector( UT_Vector * vp)
  */
 void AP_Dialog_Lists::fillDialogFromBlock(void)
 {
-	UT_Vector va,vp;
+	UT_GenericVector<const XML_Char*> va,vp;
 
 	if (getBlock()->getPreviousList() != NULL)
 	{
@@ -743,6 +790,22 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 	}
 	getBlock()->getListAttributesVector( &va);
 	getBlock()->getListPropertyVector( &vp);
+
+//
+// First get the fold level.
+//
+	const PP_AttrProp * pAP = NULL;
+	getBlock()->getAP(pAP);
+	const XML_Char *pszTEXTFOLDED = NULL;
+	if(!pAP || !pAP->getProperty("text-folded",pszTEXTFOLDED))
+	{
+		m_iCurrentLevel = 0;
+	}
+	else
+	{
+		m_iCurrentLevel = atoi(pszTEXTFOLDED);
+	}
+	setFoldLevelInGUI();
 	//
 	// First do properties.
 	//
@@ -752,7 +815,7 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		i = findVecItem(&vp,"start-value");
 		if(i >= 0)
 		{
-			m_iStartValue = atoi( (const XML_Char *) vp.getNthItem(i+1));
+			m_iStartValue = atoi(vp.getNthItem(i+1));
 		}
 		else
 		{
@@ -762,7 +825,7 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		i = findVecItem(&vp,"margin-left");
 		if(i>=0)
 		{
-			m_fAlign = (float)UT_convertToInches((const XML_Char *) vp.getNthItem(i+1));
+			m_fAlign = (float)UT_convertToInches(vp.getNthItem(i+1));
 		}
 		else
 		{
@@ -772,7 +835,7 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		i = findVecItem(&vp,"text-indent");
 		if(i >= 0)
 		{
-			m_fIndent = (float)UT_convertToInches((const XML_Char *) vp.getNthItem(i+1));
+			m_fIndent = (float)UT_convertToInches(vp.getNthItem(i+1));
 		}
 		else
 		{
@@ -791,11 +854,11 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		}
 		else if(i >=0 )
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) vp.getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, vp.getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, (const XML_Char *) "%L");
+			UT_XML_strncpy( (XML_Char *) m_pszDelim, 80, "%L");
 		}
 
 
@@ -811,26 +874,26 @@ void AP_Dialog_Lists::fillDialogFromBlock(void)
 		}
 		else if( i>= 0)
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) vp.getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, vp.getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, (const XML_Char *) ".");
+			UT_XML_strncpy( (XML_Char *) m_pszDecimal, 80, ".");
 		}
 
 		i = findVecItem(&vp,"field-font");
 		if( i>= 0)
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) vp.getNthItem(i+1));
+			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, vp.getNthItem(i+1));
 		}
 		else
 		{
-			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, (const XML_Char *) "NULL");
+			UT_XML_strncpy( (XML_Char *) m_pszFont, 80, "NULL");
 		}
 		i = findVecItem(&vp,"list-style");
 		if( i>= 0)
 		{
-			m_DocListType = getBlock()->getListTypeFromStyle( (const XML_Char *) vp.getNthItem(i+1));
+			m_DocListType = getBlock()->getListTypeFromStyle(vp.getNthItem(i+1));
 		}
 		else
 		{
@@ -931,7 +994,7 @@ UT_uint32 AP_Dialog_Lists::getID(void)
  * This method returns the index to the value corresponding to the
  * key in this props vector
  */
-UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, char * key)
+UT_sint32  AP_Dialog_Lists::findVecItem(UT_GenericVector<const XML_Char*> * v, char * key)
 {
 	const char* const_key = key;
 	return findVecItem( v, const_key);
@@ -941,7 +1004,7 @@ UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, char * key)
  * This method returns the index to the value corresponding to the
  * key in this props vector
  */
-UT_sint32  AP_Dialog_Lists::findVecItem(UT_Vector * v, const char * key)
+UT_sint32  AP_Dialog_Lists::findVecItem(UT_GenericVector<const XML_Char*> * v, const char * key)
 {
 	UT_sint32 i = v->getItemCount();
 	if(i < 0)
@@ -975,7 +1038,11 @@ void AP_Dialog_Lists::ConstructWindowName(void)
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	XML_Char * tmp = NULL;
 	UT_uint32 title_width = 33;
-	UT_XML_cloneNoAmpersands(tmp, pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Title).utf8_str());
+
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Lists_Title,s);
+	
+	UT_XML_cloneNoAmpersands(tmp, s.utf8_str());
 	BuildWindowName((char *) m_WindowName,(char*)tmp,title_width);
 	FREEP(tmp);
 }
@@ -988,14 +1055,22 @@ void AP_Dialog_Lists::setActiveFrame(XAP_Frame *pFrame)
 
 bool AP_Dialog_Lists::setView(FV_View * view)
 {
-	m_pView = (FV_View *) getActiveFrame()->getCurrentView();
+	if (getActiveFrame())
+		m_pView = (FV_View *) getActiveFrame()->getCurrentView();
+	else
+		m_pView = NULL;
+
 	return true;
 }
 
 FV_View * AP_Dialog_Lists::getView(void) const
 {
 	XAP_Frame * pFrame = getActiveFrame();
-	return (FV_View *) pFrame->getCurrentView();
+
+	if (pFrame)
+		return (FV_View *) pFrame->getCurrentView();
+	else
+		return NULL;
 }
 
 AV_View * AP_Dialog_Lists::getAvView(void)
@@ -1026,7 +1101,6 @@ AP_Lists_preview::AP_Lists_preview(GR_Graphics * gc, AP_Dialog_Lists * pLists)
 
 AP_Lists_preview::~AP_Lists_preview()
 {
-	DELETEP(m_pFont);
 }
 
 AP_Dialog_Lists * AP_Lists_preview::getLists(void)
@@ -1047,7 +1121,7 @@ void AP_Lists_preview::setData(XML_Char * pszFont,float fAlign,float fIndent)
 	{
 		m_pFont = m_gc->findFont((char *)pszFont, "normal", "", "normal", "", "16pt");
 	}	
-	UT_ASSERT(m_pFont);
+	UT_ASSERT_HARMLESS(m_pFont);
 	
 	m_fAlign = fAlign;
 	m_fIndent = fIndent;
@@ -1057,6 +1131,9 @@ void AP_Lists_preview::setData(XML_Char * pszFont,float fAlign,float fIndent)
 void AP_Lists_preview::draw(void)
 {
 	UT_return_if_fail(m_pFont);
+
+	GR_Painter painter(m_gc);
+
 	m_gc->setFont(m_pFont);
 	
 	UT_RGBColor clrGrey = UT_RGBColor(128,128,128);
@@ -1074,7 +1151,7 @@ void AP_Lists_preview::draw(void)
 	//
 	if (m_bFirst == true)
 	{
-		m_gc->clearArea(0, 0, iWidth, iHeight);
+		painter.clearArea(0, 0, iWidth, iHeight);
 	}
 	m_gc->setColor(clrBlack);
 	UT_sint32 yoff = m_gc->tlu(5) ;
@@ -1114,7 +1191,7 @@ void AP_Lists_preview::draw(void)
 
 			len = UT_UCS4_strlen(ucs_label);
 			yloc = yoff + iAscent + (iHeight - 2*yoff -iFont)*i/4;
-			//    m_gc->drawChars(ucs_label,0,len,xoff+indent,yloc);
+			//    painter.drawChars(ucs_label,0,len,xoff+indent,yloc);
 			twidth = m_gc->measureString(ucs_label,0,len,NULL);
 			if(twidth > maxw)
 				maxw = twidth;
@@ -1151,14 +1228,14 @@ void AP_Lists_preview::draw(void)
 	// Now finally draw the preview
 	//
 
-	FriBidiCharType iDirection = getLists()->getBlock()->getDominantDirection();
+	UT_BidiCharType iDirection = getLists()->getBlock()->getDominantDirection();
 
 	for(i=0; i<8; i++)
 	{
 		//
 		// First clear the line
 		//
-		m_gc->clearArea(0, m_iLine_pos[i], iWidth, iHeight);
+		painter.clearArea(0, m_iLine_pos[i], iWidth, iHeight);
 		if((i & 1) == 0)
 		{
 			//
@@ -1169,33 +1246,11 @@ void AP_Lists_preview::draw(void)
 
 			if(lv != NULL)
 			{
-				len = UT_MIN(UT_UCS4_strlen(lv),51);
+				len = UT_MIN(UT_UCS4_strlen(lv),49);
 
 				if(len > 1 && XAP_App::getApp()->theOSHasBidiSupport() == XAP_App::BIDI_SUPPORT_GUI)
 				{
-					FriBidiChar * fLogStr = new FriBidiChar[len+1];
-					FriBidiChar * fVisStr = new FriBidiChar[len+1];
-					UT_ASSERT(fLogStr && fVisStr);
-
-					for(j=0; j<=len;j++)
-						fLogStr[j] = lv[j];
-
-					fribidi_log2vis(/* input */
-			    		 fLogStr,
-					     len,
-					     &iDirection,
-					     /* output */
-					     fVisStr,
-					     NULL,
-					     NULL,
-					     NULL
-					     );
-
-					for(j=0; j<=len;j++)
-						ucs_label[j] = (UT_UCSChar)fVisStr[j];
-
-					delete [] fLogStr;
-					delete [] fVisStr;
+					UT_bidiReorderString(lv, len, iDirection, ucs_label);
 				}
 				else
 				{
@@ -1207,28 +1262,28 @@ void AP_Lists_preview::draw(void)
 				len = UT_UCS4_strlen(ucs_label);
 				yloc = yoff + iAscent + (iHeight - 2*yoff -iFont)*i/8;
 
-				if(iDirection == FRIBIDI_TYPE_RTL)
-					m_gc->drawChars(ucs_label,0,len,iWidth - xoff - indent - maxw,yloc);
+				if(iDirection == UT_BIDI_RTL)
+					painter.drawChars(ucs_label,0,len,iWidth - xoff - indent - maxw,yloc);
 				else
-					m_gc->drawChars(ucs_label,0,len,xoff+indent,yloc);
+					painter.drawChars(ucs_label,0,len,xoff+indent,yloc);
 
 				yy = m_iLine_pos[i];
 				awidth = iWidth - 2*xoff - xy;
 
-				if(iDirection == FRIBIDI_TYPE_RTL)
-					m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+				if(iDirection == UT_BIDI_RTL)
+					painter.fillRect(clrGrey,xoff,yy,awidth,aheight);
 				else
-					m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
+					painter.fillRect(clrGrey,xy,yy,awidth,aheight);
 			}
 			else
 			{
 				yy = m_iLine_pos[i];
 				awidth = iWidth - 2*xoff - xy;
 
-				if(iDirection == FRIBIDI_TYPE_RTL)
-					m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+				if(iDirection == UT_BIDI_RTL)
+					painter.fillRect(clrGrey,xoff,yy,awidth,aheight);
 				else
-					m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
+					painter.fillRect(clrGrey,xy,yy,awidth,aheight);
 			}
 		}
 		else
@@ -1236,10 +1291,10 @@ void AP_Lists_preview::draw(void)
 			yy = m_iLine_pos[i];
 			awidth = iWidth - 2*xoff - xx;
 
-			if(iDirection == FRIBIDI_TYPE_RTL)
-				m_gc->fillRect(clrGrey,xoff,yy,awidth,aheight);
+			if(iDirection == UT_BIDI_RTL)
+				painter.fillRect(clrGrey,xoff,yy,awidth,aheight);
 			else
-				m_gc->fillRect(clrGrey,xy,yy,awidth,aheight);
+				painter.fillRect(clrGrey,xy,yy,awidth,aheight);
 		}
 	}
 }
