@@ -499,6 +499,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_TOCOK)
   {
     s = EV_MIS_Gray;
   }
+  else if(pView->getFrameEdit()->isActive())
+  {
+    s = EV_MIS_Gray;
+  }
   else if(pView->isInFrame(pView->getSelectionAnchor()))
   {
     s = EV_MIS_Gray;
@@ -1483,6 +1487,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_InFootnote)
 	{
 		return EV_MIS_Gray;
 	}
+	else if(pView->getFrameEdit()->isActive())
+	{
+	        return EV_MIS_Gray;
+	}
 	if(!pView->isInFootnote() && !pView->isHdrFtrEdit() && !pView->isInHdrFtr(pView->getPoint()) && !pView->isInFrame(pView->getPoint()) 
 		&& !pView->isTOCSelected())
 	{
@@ -1548,6 +1556,10 @@ Defun_EV_GetMenuItemState_Fn(ap_GetState_BreakOK)
 	{
 		return EV_MIS_Gray;
 	}
+	else if(pView->getFrameEdit()->isActive())
+	{
+	        return EV_MIS_Gray;
+	}
 	else if(pView->isInTable(pView->getSelectionAnchor()))
 	{
 		return EV_MIS_Gray;
